/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: AbstractListModel.java,v 1.5 2002/06/11 01:19:02 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.enhydra.barracuda.core.comp.model.*;

/**
 * This class provides the abstract implementation
 * for a List Model.
 */
public abstract class AbstractListModel implements ListModel {

    protected ViewContext viewContext = null;
    protected List listeners = new ArrayList();

    //--------------- AbstractTemplateModel ----------------------
    /**
     * Add a listener to the template that's notified each time a change
     * to the data model occurs.
     *
     * @param ml the TemplateModelListener
     */
    public void addModelListener(ModelListener ml) {
        listeners.add(ml);
    }

    /**
     * Remove a listener
     *
     * @param ml the TemplateModelListener
     */
    public void removeModelListener(ModelListener ml) {
        listeners.remove(ml);
    }
    
    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     */
    public void fireModelChanged() {
        Iterator it = listeners.iterator();
        ModelListener ml = null;
        while (it.hasNext()) {
            ml = (ModelListener) it.next();
            ml.modelChanged(this);
        }
    }
//    protected List listeners = new ArrayList();

    //--------------- AbstractListModel --------------------------
    /**
     * Add a listener to the list that's notified each time a change
     * to the data model occurs.
     *
     * @param l the ListDataListener
     */  
//    public void addListDataListener(ListDataListener l) {
//        listeners.add(l);
//    }

    /**
     * Remove a listener from the list that's notified each time a 
     * change to the data model occurs.
     * @param l the ListDataListener
     */  
//    public void removeListDataListener(ListDataListener l) {
//        listeners.remove(l);
//    }

    /**
     * Reset the model to its initial (unprocessed) state. This 
     * is a convenience method that gets invoked prior to the 
     * entire model being rendered. You only need to override this 
     * method if you want to do something (like reset internal counters)
     * before the model is queried
     */
    public void resetModel() {
        //nop    
    }

    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements of the list change.  The changed elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireContentsChanged(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements are added to the model.  The new elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireIntervalAdded(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements are removed from the model.  The new elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireIntervalRemoved(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * Return all the listeners for this given list model (returns 
     * a copy of the underlying List)
     *
     * @returns all the listeners for this given list model
     */
/*
    public List getListeners() { 
        return new ArrayList(listeners); 
    }
*/

    //--------------- Contextual ---------------------------------
    /**
     * Specify the ViewContext. This method will generally be called
     * by the class that is using the model to actually render the data
     * in a view. The context will be specified prior to a render pass,
     * and the context will be reset to null after the render pass. 
     *
     * @param vc the current ViewContext
     */
    public void setViewContext(ViewContext ivc) {
        viewContext = ivc;
    }

    /**
     * Get the current ViewContext
     *
     * @return the current ViewContext
     */
    public ViewContext getViewContext() {
        return viewContext;
    }

}