/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Christian Cryder, Diez B. Roggisch, Jacob Kjome
 *
 * $Id: BAction.java,v 1.29 2003/08/12 04:42:17 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.events.*;
import org.enhydra.barracuda.core.util.http.URLRewriter;


/**
 * A BAction component is a component that you can use to catch
 * client side events. It can be used to render &lt;a&gt;, &lt;form&gt;, &lt;button&gt;,
 * &lt;input&gt;, and &lt;select&gt; elements. It allow you to specify a specific 
 * action to be generated, or can simply default to what's in the
 * template markup. You can also add event listeners directly to the 
 * component, and only those particular listeners will be notified when
 * the event actually occurs on the client.
 *
 * <p>In most cases you will not actually need to bind the component
 * to a view in order to use it--if you return it from a model, this
 * will be done for you automatically. If however, you intend to use
 * the component <em>standalone</em> (ie. manually attaching it to a 
 * specific node in the DOM) or <em>inline</em> (ie. in a toString()), 
 * then you MUST BIND IT TO A VIEW before rendering, or an error will 
 * be generated.
 */
public class BAction extends BComponent {

    //public vars
    protected static final Logger logger = Logger.getLogger(BAction.class.getName());
    
    //private vars
    protected ControlEvent actionEvent = null;
    protected String actionUrl = null;
    protected List listeners = null;
    protected Map params = null;
    protected Collection scriptFunctions = null; //ideally, implementation should provide Set-like functionality
    protected boolean disableBackButton = false;
    
    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BAction() {}
    
    /**
     * Public constructor which creates the component and binds it
     * to a view. Also allows you to specify that a custom url be fired.
     *
     * @param iactionUrl the action url to be fired
     */
    public BAction(String iactionUrl) {
        if (iactionUrl!=null) this.setAction(iactionUrl);
    }
    
    /**
     * Public constructor which creates the component and binds it
     * to a view. Also allows you to specify that a custom event be fired.
     *
     * @param iactionEvent the action event to be fired
     */
    public BAction(ControlEvent iactionEvent) {
        if (iactionEvent!=null) this.setAction(iactionEvent);
    }


    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BAction.class, HTMLElement.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLActionRenderer();}
    }


    //--------------- BAction ------------------------------------
    /**
     * Set the action to be fired by this component.
     *
     * @param iactionUrl the URL representing the action to be fired
     */
    public void setAction(String iactionUrl) {
        this.actionUrl = iactionUrl;
        this.invalidate();
    }
    
    /**
     * Set the action to be fired by this component.
     *
     * @param iactionEvent the event to be fired by this component
     */
    public void setAction(ControlEvent iactionEvent) {
        this.actionEvent = iactionEvent;
        this.invalidate();
    }
    
    //csc_041403.1 - added
    /**
     * Check for an existing action
     *
     * @return true if either the action (either URL or Event) has been set
     */
    public boolean hasAction() {
        return (this.actionUrl!=null || this.actionEvent!=null || (this.listeners!=null && this.listeners.size()>0));
    }
    
    /**
     * Get the action to be fired by this component. The action
     * will either be the URL or the Event that backs this component
     *
     * @param vc the ViewContext that determines the context of the action
     * @return a string representing the action to be fired by this component
     */
    public String getAction(ViewContext vc) {
        return this.getAction(vc, false);
    }
    
    //csc_101701.1 - this method (with additional parameter) added
    /**
     * Get the action to be fired by this component. The action
     * will either be the URL or the Event that backs this component
     *
     * @param ivc the ViewContext that determines the context of the action
     * @param preventRewriting this should be set to true only if you want to
     *      prevent URLRewriting from taking place. As a developer, you 
     *      will probably never need to do this, unless to need to further
     *      modify the action prior to finally encoding it (HTMLActionRenderer
     *      does this)
     * @return a string representing the action to be fired by this component
     */
    public String getAction(ViewContext vc, boolean preventRewriting) {
        StringBuffer sb = new StringBuffer(200);
        String sep = "?";

        //build the base action
        if (this.actionUrl!=null) {
            //from the url
            if (preventRewriting) sb.append(this.actionUrl);
            else sb.append(URLRewriter.encodeURL(vc, this.actionUrl));    //take into account the need for URL rewriting
        } else {
            //from the event
            if (this.actionEvent==null) this.setAction(new ActionEvent());

            if (preventRewriting) sb.append(this.actionEvent.getEventIDWithExtension());
            else sb.append(URLRewriter.encodeURL(vc, this.actionEvent.getEventIDWithExtension()));    //take into account the need for URL rewriting

            //now, see if we have event listeners
            if (this.listeners!=null && this.listeners.size()>0) {
                Iterator it = this.listeners.iterator();
                while (it.hasNext()) {
                    ListenerFactory lf = (ListenerFactory) it.next();
                    sb.append(sep).append(BaseEvent.EVENT_ID).append("=").append(lf.getListenerID());
                    sep = "&";
                }
            }                
        }

        //finally add any custom params
        if (this.params!=null && this.params.size()>0) {
            Iterator it = this.params.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object val = this.params.get(key);
                if (key!=null && val!=null) {
                    if (val instanceof String[]) {
                        String [] vals = (String [])val;
                        for (int i = 0; i < vals.length; i++) {
                            sb.append(sep);
                            sb.append(URLEncoder.encode(key.toString()));
                            sb.append("=");
                            sb.append(URLEncoder.encode(vals[i].toString()));
                            sep = "&";
                        }
                    } else {
                        sb.append(sep);
                        sb.append(URLEncoder.encode(key.toString()));
                        sb.append("=");
                        sb.append(URLEncoder.encode(val.toString()));
                    }
                }
                sep = "&";
            }
        }
        return sb.toString();
    }
    
    /**
     * Set any associated params
     */
    public void setParam(String key, String val) {
        if (this.params==null) this.params = new HashMap(10);
        this.params.put(key, val);
    }

    //dbr_012202.2 - added
    /**
     * Set any associated list of params 
     */
    public void setParam(String key, String[] val) {
        if (this.params==null) this.params = new HashMap(10);
        this.params.put(key, val);
    }
    
    /**
     * Get any associated params
     */
    public Map getParams() {
        return this.params;
    }

    /**
     * stores a Set of client-side script functions for use by the
     * {@link org.enhydra.barracuda.core.comp.renderer.html.HTMLActionRenderer}
     * .  These script functions are javascript function names.  The
     * constraints for these functions is that they take a form element as an
     * argument (or no arguments) and that it returns a boolean.  These
     * functions are called in the order given and can be used for custom
     * client-side form validation.
     * 
     * @since 1.2
     * @param functionName the name of the javascript function with no
     *        parenthasis
     */
    public void addScriptFunction(String functionName) {
        if (this.scriptFunctions==null) this.scriptFunctions = new ArrayList(5); //jrk_20030501 - I'd rather use j2sdk1.4+ LinkedHashSet here to preserve original added order!!!
        if (!this.scriptFunctions.contains(functionName)) { //unnecessary if using a LinkedHashSet
            this.scriptFunctions.add(functionName);
        }
    }

    /**
     * provides access any custom script functions added to a BAction component
     * 
     * @since 1.2
     * @see #addScriptFunction(String)
     * @return a Set of script functions or null if none added
     */
    public Collection getScriptFunctions() {
        return this.scriptFunctions;
    }

    /**
     * Set disable back button (only works if your client supports
     * Javascript)
     *
     * @param disable true if we want the back button disabled 
     */
    public void setDisableBackButton(boolean idisableBackButton) {
        this.disableBackButton = idisableBackButton;
    }

    /**
     * Get disable back button
     *
     * @return true if we want the back button disabled 
     */
    public boolean getDisableBackButton() {
        return this.disableBackButton;
    }

    /**
     * Add an event listener to this component.
     *
     * @param lf the event listener to be added
     */
    public void addEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (this.listeners==null) this.listeners = new ArrayList(5);
        this.listeners.add(lf);    
        this.invalidate();
    }
    
    /**
     * Remove an event listener from this component
     *
     * @param lf the event listener to be removed
     */
    public void removeEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (this.listeners==null) return;
        this.listeners.remove(lf);    
        this.invalidate();
    }

}
