/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s): Christian Cryder, Stephen Peterson, Jacob Kjome
 * 
 * $Id: DefaultView.java,v 1.11 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;

import org.enhydra.barracuda.core.util.dom.DOMUtil;

/**
 * This interface defines a View for components to 
 * render themselves in. Similar to the java.awt.Graphics 
 * object in AWT and Swing.
 * 
 * @author  Christian Cryder <christianc@granitepeaks.com>
 * @author  Stephen Peterson <stephen_peterson@agilent.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public class DefaultView implements View {

    protected static final Logger logger = Logger.getLogger(DefaultView.class.getName());

    private final static String DEFAULT = "~DEFAULT~";

    protected Node node = null;
    private String name = null;
    protected Map templateNodes = null;
    protected LocalElementFactory localFactory = null;
    
    //--------------- Constructors -------------------------------
    /**
     * Default constructor to create a view
     */
    public DefaultView() {
        this (null, null);
    }

    /**
     * Create a view and bind it to a node
     *
     * @param name the name of the view
     */
    public DefaultView(String name) {
        this (name, null);
    }

    /**
     * Create a view and bind it to a node
     *
     * @param node the node to which the view is bound
     */
    public DefaultView(Node node) {
        this (null, node);
    }

    /**
     * Create a view and bind it to a node
     *
     * @param name the name of the view
     * @param node the node to which the view is bound
     */
    public DefaultView(String name, Node node) {
        if (name!=null) setName(name);
        if (node!=null) setNode(node);
    }



    //--------------- DefaultView --------------------------------
    /**
     * Bind the view to a specific DOM node
     *
     * @param node the specific DOM node to bind the View to
     */
    public void setNode(Node inode) {
        //set the node
        if (logger.isDebugEnabled()) logger.debug("Setting node in "+this+" to:"+node);
        node = inode;
        
        //serach for templates associated with the node
        searchForTemplates(node);
    }
    
    /**
     * Get the specific DOM node the View is bound to
     *
     * @return the specific DOM node the View is bound to
     */
    public Node getNode() {
        return node;    
    }
    
    /**
     * Set the name for this view
     *
     * @param name the name for this view
     */
    public void setName(String iname) {
        name = iname;
    }
    
    /**
     * Get the name for this view
     *
     * @return the name for this view
     */
    public String getName() {
        if (name==null) name = "@"+Integer.toHexString(this.hashCode());
        return name;
    }
        
    /**
     * Get the element factory provided by this view
     *
     * @return the element factory provided by this view
     */    
    public ElementFactory getElementFactory() {
        if (localFactory==null) localFactory = new LocalElementFactory();
        return localFactory;
    }

    /**
     * Get a String describing the view
     */
    public String toString() {
        return "View:"+getName()+" (bound to Node:"+DOMUtil.getID(node)+")";
    }

    /**
     * Given a specific node, find any templates associated with it or its
     * children.
     */
    protected void searchForTemplates(Node curnode) {
        if (curnode==null) {
            // Some (incorrect?) setups come here with curnode==null. Make this safe.  -srp Agilent Technologies 11/12/2001
            logger.warn("The current node in which templates are to be searched is null");
            return;
        }

        if (logger.isDebugEnabled()) logger.debug("Looking for templates for Node:"+curnode+"...");

        //first allow for any custom searching for templates
        customSearchForTemplates(curnode);                

        //next see if we can find any template nodes
        if (curnode.hasChildNodes()) {
            NodeList nl = curnode.getChildNodes();
            
            //iterate through the children
            for (int i=0,max=nl.getLength(); i<max; i++) {
                Node child = nl.item(i);
                if (logger.isDebugEnabled()) logger.debug("Found child:"+child);

                //if its not an Element skip it
                if (child instanceof Element) {
                    //create the template nodes if necessary
                    if (templateNodes==null) templateNodes = new HashMap();
                    
                    //set the default template value (the first one we find)
                    if (templateNodes.get(DEFAULT)==null) {
                        if (logger.isDebugEnabled()) logger.debug("Setting default in xref!");
                        templateNodes.put(DEFAULT, child);
                    }
                    
                    //062501_ba_start - Bob Anderson - 
                    //
                    //The effect of this addition is to allow one to do an operation such as
                    //    form = (HTMLFormElement) elementFactory.getElement(MyXMLCObject.NAME_MyFormName) ;
                    //where there is a line like the following in your HTML source:
                    //    <form name="MyFormName">
                    //The current ElementFactory (as of 6-6-2001) only allows you to get
                    //elements defined by an ID attribute or by tag. The ID attribute is
                    //sometimes too restrictive since only one element in an entire HTML
                    //document may use a particular ID. "name" attributes may be repeated in a
                    //document, and they are very handy for identifying various elements that
                    //you may wish to get at different places in your document. Tags may be
                    //useful sometimes, but are probably repeated too often to be of much
                    //value.
                    //
//added                    
                    //add the child's name (if it exists)
                    String childName = ((Element) child).getAttribute("name") ;
                    if (childName!=null && !childName.equals("") && !templateNodes.containsKey(childName)) {
                        if (logger.isDebugEnabled()) logger.debug("Adding child name in xref:"+childName);
                        templateNodes.put(childName, child);
                        templateNodes.put(childName.toUpperCase(), child);
                        templateNodes.put(childName.toLowerCase(), child);
                    }

                    //This is simply a compromise for performance. My idea here is that
                    //accessing elements by tag name is rarely useful, as tags are so often
                    //repeated. Consider a table element. There may be many many TD tags.
                    //Which one will be returned by the ElementFactory, and how useful will it
                    //be? Also, since we've implemented suggestion #1 (above) this code would 
                    //be useless overhead. 
/*                    
//removed
                    //add the child's tag name
                    String tagName = ((Element) child).getTagName();
                    if (!templateNodes.containsKey(tagName)) {
                        if (logger.isDebugEnabled()) logger.debug("Adding tag name in xref:"+tagName);
                        templateNodes.put(tagName.toLowerCase(), child);
                        templateNodes.put(tagName.toUpperCase(), child);
                    }

*/
                    //062501_ba_finish

                    //add the child's id name (if it exists)
                    String idName = ((Element) child).getAttribute("id");
                    if (idName!=null && !templateNodes.containsKey(idName)) {
                        if (logger.isDebugEnabled()) logger.debug("Adding id name in xref:"+idName);
                        templateNodes.put(idName, child);
                        templateNodes.put(idName.toUpperCase(), child);
                        templateNodes.put(idName.toLowerCase(), child);
                    }

                    //add the child's class name
                    String clname = child.getClass().getName();
                    if (!templateNodes.containsKey(clname)) {
                        if (logger.isDebugEnabled()) logger.debug("Adding class name in xref:"+clname);
                        templateNodes.put(clname, child);

                        //add any interfaces the child implements
                        Class[] cl = child.getClass().getInterfaces();
                        for (int j=0,jmax=cl.length; j<jmax; j++) {
                            clname = cl[j].getName();
                            if (!templateNodes.containsKey(clname)) {
                                if (logger.isDebugEnabled()) logger.debug("Adding interface name in xref:"+clname);
                                templateNodes.put(clname, child);
                            }
                        }            
                    }
                }
                                
                //search for templates in the child
                searchForTemplates(child);
                
                if (logger.isDebugEnabled()) logger.debug("Finished check on child!");
            }
        }
    }

    /**
     * If you want to identify templates based on some custom mechanism,
     * this is the method to override. Look at DefaultTableView for an
     * example...
     */
    protected void customSearchForTemplates(Node curnode) {
        //nop
    }


    /**
     * By default, the text component template factory only returns
     * a Document; the other methods always return null.
     */
    class LocalElementFactory implements ElementFactory {
    
        public Node getDefaultElement() {
            if (templateNodes==null) return null;
            return (Node) templateNodes.get(DEFAULT);
        }

        public Node getElement(String key) {
            if (templateNodes==null) return null;
            return (Node) templateNodes.get(key);
        }
        
        public List getElementKeys() {
            if (templateNodes==null) return null;
            return new ArrayList(templateNodes.keySet());
        }

        public Document getDocument() {
            return node.getOwnerDocument();        
        }
    }

    //--------------- Cloneable ----------------------------------
    //csc_092101.1_start
    /**
     * When a view is cloned, the underlying node that backs the view is
     * set to null; you MUST bind the newly cloned view to a node before 
     * you can use it.
     */
    public Object clone() {
        try {
            DefaultView dv = (DefaultView) super.clone();
            dv.node = null;
            return dv;
       } catch (Exception e) {
            return null;
       }
    }
    //csc_092101.1_end
}