/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: HTMLActionRenderer.java,v 1.39 2003/07/31 15:47:18 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.scripting.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.http.URLRewriter;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.plankton.http.LightweightURL;

/**
 * This class handles the default rendering of actions into an HTML view.
 */
public class HTMLActionRenderer extends HTMLComponentRenderer {

    protected static final Logger logger = Logger.getLogger(HTMLActionRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();

    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. In the case of BAction,
     * it will attempt to use the default template node if possible (ie. if
     * it is an <a>, <form>, <input>, <button>, or <select> element); if anything else,
     * it will defer to the super() method
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component to be rendered
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if ((templateNode instanceof HTMLAnchorElement)
         || (templateNode instanceof HTMLFormElement)
         || (templateNode instanceof HTMLInputElement)
         || (templateNode instanceof HTMLButtonElement)
         || (templateNode instanceof HTMLSelectElement)) {
            defaultNode = templateNode.cloneNode(true);
        } else {
            return super.createDefaultNode(doc, comp, vc);
        }
        if (logger.isInfoEnabled()) logger.info("Creating default node: "+defaultNode);
        return defaultNode;
    }

    /**
     * Render the data from the component into the view, taking into
     * consideration the specified ViewContext
     *
     * @param comp the component to be rendered
     * @param view the view the component should be rendered in
     * @param vc the view context
     * @throws RenderException if unable to render the component in the 
     *        specified view
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a BAction component
        if (!(comp instanceof BAction)) {
            String errmsg = "This renderer can only render BAction components: "+comp;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
        BAction actionComp = (BAction) comp;
        HTMLElement el = (HTMLElement)view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(actionComp, view, vc);

        manipulateActionElement(el, actionComp, vc);

        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(el, actionComp.isEnabled());
    }

    /**
     * Generic Element - cast elements to more specific elements in order to
     * call the appropriate overloaded method.  Throw an exception if the
     * element is not supported.
     *
     * @param el the HTML element to be manipulated
     * @param comp the BAction component containing information about the
     *        action to take on bound HTML element
     * @param vc the current ViewContext
     * @throws RenderException
     */
    protected static void manipulateActionElement(Element el, BAction comp, ViewContext vc) throws RenderException {
        if (el instanceof HTMLAnchorElement) {
            manipulateActionElement((HTMLAnchorElement)el, comp, vc);
        } else if (el instanceof HTMLFormElement) {
            manipulateActionElement((HTMLFormElement)el, comp, vc);
        } else if (el instanceof HTMLInputElement) {
            manipulateActionElement((HTMLInputElement)el, comp, vc);
        } else if (el instanceof HTMLButtonElement) {
            manipulateActionElement((HTMLButtonElement)el, comp, vc);
        } else if (el instanceof HTMLSelectElement) {
            manipulateActionElement((HTMLSelectElement)el, comp, vc);
        //} else if (el instanceof [another supported element here]) {
        //    manipulateActionElement(([another supported element here])el, comp, vc);
        //}
        } else {
            String errmsg = "Element does not implement one of HTMLAnchorElement, HTMLFormElement, HTMLInputElement, HTMLButtonElement, or HTMLSelectElement and cannot be rendered: "+el;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }

    /**
     * HTMLAnchorElement - set the "href" attribute
     *
     * @see #manipulateActionElement(Element, BAction, ViewContext)
     */
    protected static void manipulateActionElement(HTMLAnchorElement el, BAction comp, ViewContext vc) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAnchorElement interface...");

        //set the href
 //csc_041403.1 - basically, don't set the action unless its actually been specified in the component;
 //we may still need to do this in the other manipulateActionElementFunctions...
 //csc_041403.1        el.setHref(comp.getAction(vc));
        if (comp.hasAction()) el.setHref(comp.getAction(vc));   //csc_041403.1

        //if we need to disable the back button and the client supports
        //Javascript, we can do so by setting the onclick attribute
        if (comp.getDisableBackButton() && clientScriptingAllowed(comp, vc)) {
            el.setAttribute("onclick", "location.replace(this.href);return false;");
        }
    }

    /**
     * HTMLFormElement - set the "action" attribute
     *
     * @see #manipulateActionElement(Element, BAction, ViewContext)
     */
    protected static void manipulateActionElement(HTMLFormElement el, BAction comp, ViewContext vc) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLFormElement interface...");

        //if the client supports Javascript...
        if (clientScriptingAllowed(comp, vc)) {
            StringBuffer sb = new StringBuffer(200);
            StringBuffer sbAction = new StringBuffer(200);
            boolean disable = comp.getDisableBackButton();

            if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
            LightweightURL lu = new LightweightURL(comp.getAction(vc, true));
            sbAction.append(lu.getBaseStr());
            if (disable) sbAction.append(ParamGateway.PARAM_EXT);
            sbAction.append(lu.getParamStr());

            //redirect the form to param gateway servlet
            el.setAction(URLRewriter.encodeURL(vc, sbAction.toString()));

            addSubmitScript(comp, sb, "this");

            //set the onsubmit attribute
            el.setAttribute("onsubmit", sb.toString());

            //make sure we include the necessary script library
            includeScriptLibrary(el, comp, BScriptResource.JS_FORM_CONTROL);
            if (disable) includeScriptLibrary(el, comp, BScriptResource.JS_CLIENT_SERVER_HTTP_LIB);
        } else { //else just set the form action
            el.setAction(comp.getAction(vc));
        }
    }

    /**
     * HTMLInputElement
     *
     * @see #manipulateActionElement(Element, BAction, ViewContext)
     */
    protected static void manipulateActionElement(HTMLInputElement el, BAction comp, ViewContext vc) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
        HTMLFormElement fel = el.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)

        //if the client supports Javascript...
        if (clientScriptingAllowed(comp, vc)) {
            //build the action String
            StringBuffer sb = new StringBuffer(200);
            StringBuffer sbAction = new StringBuffer(100);
            boolean disable = comp.getDisableBackButton();

            //adjust the documents action
            if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
            LightweightURL lu = new LightweightURL(comp.getAction(vc, true));
            sbAction.append(lu.getBaseStr());
            if (disable) sbAction.append(ParamGateway.PARAM_EXT);
            sb.append("this.form.action='").append(URLRewriter.encodeURL(vc, sbAction.toString())).append(lu.getParamStr()).append("';");

            addSubmitScript(comp, sb, "this.form");

            //adjust the onclick value to invoke submit
            el.setAttribute("onclick", sb.toString());

            //if we are dealing with a "submit" button, we need to set the
            //form's action as well since, if the user hits enter, the form
            //will be submitted but the button will not be clicked (IE actually
            //seems to handle this correctly, but Mozilla does not; this will
            //work for both). Thanks to Iman L. Crawford [icrawford@greatnation.com]
            //for pointing out this problem.
            if ("submit".equalsIgnoreCase(el.getType())) {
                fel.setAttribute("onsubmit", new StringBuffer(50).append("this.").append(NamingHelper.getName(el)).append(".click();return false;").toString());
            }

            //make sure we include the necessary script library
            includeScriptLibrary(el, comp, BScriptResource.JS_FORM_CONTROL);
            if (disable) includeScriptLibrary(el, comp, BScriptResource.JS_CLIENT_SERVER_HTTP_LIB);
        } else { //else do it the manual way
            //make sure the button type is submit
            if (!"submit".equalsIgnoreCase(el.getType())) {
                if ("button".equalsIgnoreCase(el.getType())) {
                    //modify the <input> type to make this form submitable in
                    //a non-javascript environment.
                    el.setAttribute("type", "submit");
                } else {
                    if (comp.getRenderStrategy() != RenderStrategy.CUSTOM_SCRIPT) {
                        //whoops, it's an <input> element of neither type "submit",
                        //nor type "button". Probably overstepping bounds in
                        //modifying the type attribute here, so throw exception
                        //instead.
                        String errmsg = "Cannot render Input action listener; input type is not 'submit': "+el;
                        logger.warn(errmsg);
                        throw new NoSuitableRendererException(errmsg);
                    }
                }
            }

            //rename the button appropriately (don't encode)
            el.setName(FormGateway.FORM_TARGET+comp.getAction(vc, true));

            //redirect the form to event forwarding servlet (encode)
            fel.setAction(URLRewriter.encodeURL(vc, NamingHelper.getName(fel)+FormGateway.FORM_EXT));
        }
    }

    /**
     * HTMLButtonElement
     *
     * @see #manipulateActionElement(Element, BAction, ViewContext)
     */
    protected static void manipulateActionElement(HTMLButtonElement el, BAction comp, ViewContext vc) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLButtonElement interface...");
        HTMLFormElement fel = el.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)

        //if the client supports Javascript...
        if (clientScriptingAllowed(comp, vc)) {
            //build the action String
            StringBuffer sb = new StringBuffer(200);
            StringBuffer sbAction = new StringBuffer(100);
            boolean disable = comp.getDisableBackButton();

            //adjust the documents action
            if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
            LightweightURL lu = new LightweightURL(comp.getAction(vc, true));
            sbAction.append(lu.getBaseStr());
            if (disable) sbAction.append(ParamGateway.PARAM_EXT);
            sb.append("this.form.action='").append(URLRewriter.encodeURL(vc, sbAction.toString())).append(lu.getParamStr()).append("';");

            addSubmitScript(comp, sb, "this.form");

            //adjust the onclick value to invoke submit
            el.setAttribute("onclick", sb.toString());

            //if we are dealing with a "submit" button, we need to set the
            //form's action as well since, if the user hits enter, the form
            //will be submitted but the button will not be clicked (IE actually
            //seems to handle this correctly, but Mozilla does not; this will
            //work for both). Thanks to Iman L. Crawford [icrawford@greatnation.com]
            //for pointing out this problem.
            if ("submit".equalsIgnoreCase(el.getType())) {
                fel.setAttribute("onsubmit", new StringBuffer(50).append("this.").append(NamingHelper.getName(el)).append(".click();return false;").toString());
            }

            //make sure we include the necessary script library
            includeScriptLibrary(el, comp, BScriptResource.JS_FORM_CONTROL);
            if (disable) includeScriptLibrary(el, comp, BScriptResource.JS_CLIENT_SERVER_HTTP_LIB);
        } else { //else do it the manual way
            //make sure the button type is submit
            if (!"submit".equalsIgnoreCase(el.getType())) {
                if ("button".equalsIgnoreCase(el.getType())) {
                    //modify the <button> type to make this form submitable in
                    //a non-javascript environment.
                    el.setAttribute("type", "submit");
                } else {
                    if (comp.getRenderStrategy() != RenderStrategy.CUSTOM_SCRIPT) {
                        //whoops, it's a "reset" <button>. Probably overstepping
                        //bounds in modifying the type attribute here, so throw
                        //exception instead.
                        String errmsg = "Cannot render Button action listener; button type is not 'submit': "+el;
                        logger.warn(errmsg);
                        throw new NoSuitableRendererException(errmsg);
                    }
                }
            }

            //rename the button appropriately (don't encode)
            el.setName(FormGateway.FORM_TARGET+comp.getAction(vc, true));

            //redirect the form to event forwarding servlet (encode)
            fel.setAction(URLRewriter.encodeURL(vc, NamingHelper.getName(fel)+FormGateway.FORM_EXT));
        }
    }

    /**
     * HTMLSelectElement
     *
     * @see #manipulateActionElement(Element, BAction, ViewContext)
     */
    protected static void manipulateActionElement(HTMLSelectElement el, BAction comp, ViewContext vc) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLSelectElement interface...");

        HTMLFormElement fel = el.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)

        //if the client supports Javascript...
        if (clientScriptingAllowed(comp, vc)) {
            //build the action String
            StringBuffer sb = new StringBuffer(200);
            StringBuffer sbAction = new StringBuffer(100);
            boolean disable = comp.getDisableBackButton();

            //adjust the documents action
            if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
            LightweightURL lu = new LightweightURL(comp.getAction(vc, true));
            sbAction.append(lu.getBaseStr());
            if (disable) sbAction.append(ParamGateway.PARAM_EXT);
            sb.append("this.form.action='").append(URLRewriter.encodeURL(vc, sbAction.toString())).append(lu.getParamStr()).append("';");

            addSubmitScript(comp, sb, "this.form");

            //set the onchange value to invoke submit
            el.setAttribute("onchange", sb.toString());

            //make sure we include the necessary script library
            includeScriptLibrary(el, comp, BScriptResource.JS_FORM_CONTROL);
            if (disable) includeScriptLibrary(el, comp, BScriptResource.JS_CLIENT_SERVER_HTTP_LIB);
        } else {
            if (comp.getRenderStrategy() != RenderStrategy.CUSTOM_SCRIPT) {
                String errmsg = "Cannot render Select action listener; client does not support Javascript: "+el;
                logger.warn(errmsg);
                throw new NoSuitableRendererException(errmsg);
            }

            //rename the button appropriately (don't encode)
            el.setName(FormGateway.FORM_TARGET+comp.getAction(vc, true));

            //redirect the form to event forwarding servlet (encode)
            fel.setAction(URLRewriter.encodeURL(vc, NamingHelper.getName(fel)+FormGateway.FORM_EXT));
        }
    }

///////////////////////// Internal Utility Functions /////////////////////////

    /**
     * Does the client support javascript, is javascript currently enabled on
     * the client, and does the current render strategy allow for javascript?
     * If so, then client scripting is allowed.
     *
     * @param comp the BAction component containing information about the
     *        action to take on bound HTML element
     * @param vc the current ViewContext
     * @return true if javascript is allowed, false if not
     */
    private static boolean clientScriptingAllowed(BAction comp, ViewContext vc) {
        return vc.getViewCapabilities().getScriptingType() instanceof ScriptingType.JavaScript1x
            && comp.getRenderStrategy() != RenderStrategy.NEVER_SCRIPT
            && comp.getRenderStrategy() != RenderStrategy.CUSTOM_SCRIPT;
    }

    /**
     * includes a script resource to the document.  Note: script resouces will
     * only be added once to the document.  Duplicates are ignored.
     *
     * @param el an HTMLElement providing a hook into the document
     * @param comp the BAction component the script resource is being assocated
     *        with
     * @param script the path to the script resource.  This is assumed to be
     *        part of the xlib resources.  An example string to pass in might
     *        be BScriptResource.JS_FORM_CONTROL
     * @see BScriptResource
     */
    private static void includeScriptLibrary(HTMLElement el, BAction comp, String script) {
        BScriptResource bsr = new BScriptResource(ResourceGateway.EXT_RESOURCE_ID+script);
        bsr.setView(new DefaultView(el));
        comp.addStepChild(bsr, true);
    }

    /**
     * adds the standard submit script for form elements which have a BAction
     * bound to them.  Also adds any script functions added to BAction.
     *
     * @param comp the BAction component being added to the document
     * @param sb the StringBuffer with which to add the script information
     * @param form the name to use for the javascript reference to the current
     *        form element.  eg... for a form element, the value would be
     *        "this".  For an element in a form, the value would be
     *        "this.form".
     * @see BAction#addScriptFunction(String)
     */
    private static void addSubmitScript(BAction comp, StringBuffer sb, String form) {
        sb.append("return doSubmitAndLock(").append(form);
        Collection scriptFunctions = comp.getScriptFunctions();
        if (scriptFunctions != null) {
            Iterator iter = scriptFunctions.iterator();
            while (iter.hasNext()) {
                sb.append(",").append((String)iter.next());
            }
        }
        if (comp.getDisableBackButton()) sb.append(",cshl_SubmitAndReplace");
        sb.append(");");
    }
}
