/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTextRenderer.java,v 1.16 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.plankton.*;

/**
 * This class handles the default rendering of text into an HTML view.
 *
 * Q: maybe rather than just setting the text in Text node, the text renderer
 * should represent the string as a CDATASection. This would enable you to 
 * easily set text as chunks of HTML. Hmmm...need to think about this...
 */
public class HTMLTextRenderer extends HTMLComponentRenderer {

    protected static final Logger logger = Logger.getLogger(HTMLTextRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
    
    /**
     * The purpose of this method is for a renderer to provide 
     * a default node (if none exists). This component currently
     * does not provide a default, so it throws an UnsupportedFormatException 
     * instead.
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = templateNode.cloneNode(true);        
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
//        throw new UnsupportedFormatException("Cannot create default node");
    }
    
    /**
     * 
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BText)) throw new NoSuitableRendererException("This renderer can only render BText components");
        BText btext = (BText) comp;
        Node node = view.getNode();
        String text = btext.getText();                                      //csc_121201.1

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //HTMLDocument Interface - set the "title" attribute
        //----------------------
        if (node instanceof HTMLDocument) {
            //..HTMLDocument - set the "title" attribute
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLDocument interface...");
            ((HTMLDocument) node).setTitle(btext.getText());
    
        //HTMLElement Interface
        //---------------------
        //Supported Elements:
        //..HTMLAreaElement - set the "alt" attribute
        //..HTMLImageElement - set the "alt" attribute
        //..HTMLInputElement - set the "value" attribute
        //..HTMLTitleElement - set the "text" attribute
        //..For everything else - get the first Text child and set 
        //....value if it exists
        } else if (node instanceof HTMLElement) {

            //..HTMLAreaElement - set the "alt" attribute
            if (node instanceof HTMLAreaElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAreaElement interface...");
//csc_121201.1                ((HTMLAreaElement) node).setAlt(btext.getText());
                if (text!=null) ((HTMLAreaElement) node).setAlt(text);   //csc_121201.1

            //..HTMLImageElement - set the "alt" attribute
            } else if (node instanceof HTMLImageElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLImageElement interface...");
//csc_121201.1                ((HTMLImageElement) node).setAlt(btext.getText());
                if (text!=null) ((HTMLImageElement) node).setAlt(text);   //csc_121201.1

            //..HTMLInputElement - set the "value" attribute
            } else if (node instanceof HTMLInputElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
//csc_121201.1                ((HTMLInputElement) node).setValue(btext.getText());
                if (text!=null) ((HTMLInputElement) node).setValue(text);   //csc_121201.1
            
            //..HTMLTitleElement - set the "text" attribute
            } else if (node instanceof HTMLTitleElement) {
                if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLTitleElement interface...");
//csc_121201.1                ((HTMLTitleElement) node).setText(btext.getText());
                if (text!=null) ((HTMLTitleElement) node).setText(text);   //csc_121201.1
            
            //..For everything else - get the first Text child and set 
            //....value if it exists
            } else {  
                if (logger.isInfoEnabled()) logger.info("Rendering based on "+Classes.getShortClassName(node.getClass())+" interface in first child Text element...");
//csc_121201.1                DOMUtil.setTextInNode((Element) node, btext.getText(), btext.allowMarkupInText());  //csc_092701.1
                if (text!=null) DOMUtil.setTextInNode((Element) node, text, btext.allowMarkupInText());  //csc_121201.1
            }
        } else {
            String errmsg = "Node does not implement HTMLElement or HTMLDocument and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
                
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
}