/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Christian Cryder, Diez Roggisch, Jacob Kjome
 *
 * $Id: ApplicationGateway.java,v 1.44 2003/06/11 15:09:06 jacobk Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.event.events.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.plankton.*;
import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.plankton.exceptions.*;
import org.enhydra.barracuda.plankton.http.URLRewriter;
import org.enhydra.barracuda.plankton.http.ServletUtil;

/**
 * <p>The application gateway is responsible for a number of things.
 *
 * <ol>
 *   <li>It acts as a gateway servlet for all event handlers in this
 *         application.</li>
 *
 *   <li>It defines and instantiates a number of entities needed to
 *       dispatch events (EventBroker, EventPool, DispatcherFactory, etc)</li>
 *
 *      <li>It defines which event extension we are using</li>
 *
 *      <li>It registers all EventGateways, and any local Event interests</li>
 *
 *      <li>It performs the initial mapping of HTTPRequests to Events for this
 *       particular domain </li>
 * </ol>
 *
 * <p>Consequently, if you want to use the Barracuda event model, this
 * is the class that really kicks it all off. You must either extend this
 * class OR use the DefaultApplicationAssembler in order to specify that
 * this servlet handles all event requests for an application. This will
 * allow the system to convert requests to events and dispatch them through
 * the EventBroker to any listeners within EventGateways that have registered
 * interest with the broker.
 *
 * <p>This class should be the first servlet loaded in your web.xml file.
 *
 * <p>For an example of how to do this, look at
 * <strong>org.enhydra.barracuda.examples.ex1.SampleApplicationGateway</strong>
 *
 * <p>You might also want to look at the <a href="sm_barracuda_event.gif">UML Class
 * diagram of the Event model classes.</a>
 *
 * @author  Christian Cryder <christianc@granitepeaks.com>
 * @author  Diez Roggisch <diez.roggisch@artnology.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public class ApplicationGateway extends HttpServlet implements EventGateway {

    protected static final Logger logger = Logger.getLogger(ApplicationGateway.class.getName());

    //public vars
    public static boolean USE_EVENT_POOLING = true;            //use event pooling?
//    public static boolean SHOW_APP_STRUCTURE = false;
    public static RequestWrapper REQUEST_WRAPPER = null;    //dbr_021602

    public static final String TARGET_EVENT_NAME = "DefaultBaseEventListener.TargetEventName";            //(String)
    public static final String EXTERNAL_CONTEXT_OBJ_NAME = "External.Context.Obj";            //(String)

//    private static final String DEBUG_VALUE = "DebugValue";
    private static final String APPLICATION_ASSEMBLER = "ApplicationAssembler";
    private static final String ASSEMBLY_DESCRIPTOR = "AssemblyDescriptor";
//    private static final String SCRIPTING_DESCRIPTOR = "ScriptingDescriptor";
    private static final String SAX_PARSER = "SAXParser";

    private EventBroker masterEventBroker = null;
    private EventPool masterEventPool = null;
//    protected DefaultEventGateway defaultEventGateway = new DefaultEventGateway();
    protected EventGateway eventGateway = (EventGateway) Classes.newInstance(A_Classes.DEFAULT_EVENT_GATEWAY);   //csc_060903_3

    //private vars
    protected List gateways = null;

    /**
     * Public noargs constructor
     */
    public ApplicationGateway() {
        logger.info("Instantiating "+this);
    }


    //--------------- User Configurable Methods ------------------
    /**
     * <p>Perform any local initialization (this is where you should
     * add any other known EventGateways)
     */
    public void initializeLocal() {
    }

    /**
     * <p>Perform any local initialization (this is where you should
     * add any other known EventGateways)
     *
     * @param config the ServletConfig object used to configure this servlet
     * @deprecated If your code is still attempting to extend this old method 
     *      signature, you need to change it to use initializeLocal() instead.
     *      If you still need to get a reference to the servlet config, just 
     *      call 'this.getServletConfig()'. This method is now final so that
     *      code which might still be using it will no longer compile.
     */
    public final void initializeLocal(ServletConfig iconfig) throws ServletException {
    }
    
    /**
     * <p>Perform any local cleanup (this is where you should
     * remove any known EventGateways)
     */
    public void destroyLocal() {
    }

    /**
     * <p>Provide an instance of the specific EventBroker we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventBroker.
     *
     * @return a new instance of the EventBroker
     */
    public EventBroker getNewEventBrokerInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventBroker");
        return new DefaultEventBroker(getDispatcherFactory(), getEventExtension());
    }
    
    /**
     * <p>Provide an instance of the specific EventPool we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventPool.
     *
     * @return a new instance of the EventPool.
     */
//csc_060903_1_start
/*
    public EventPool getNewEventPoolInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventPool");
        return new DefaultEventPool();
    }
*/
//csc_060903_1_end

    /**
     * <p>Provide an instance of the specific DispatchQueue we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultDispatchQueue.
     *
     * @return a new instance of the DispatchQueue.
     */
    public DispatchQueue getNewDispatchQueueInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatchQueue");
        return new DefaultDispatchQueue(true);
    }
    
    /**
     * <p>Provide an instance of the specific EventDispatcher we want to use.
     * Override this method if you'd like to use something other than
     * DefaultEventBroker.
     *
     * @return a new instance of the DispatcherFactory
     */
    public DispatcherFactory getDispatcherFactory() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatcherFactory");
        return new DefaultDispatcherFactory();
    }
    
    /**
     * <p>Indicate which event extension we are handling. By default the gateway
     * handles extensions of <strong>.event</strong>. If you wish to handle
     * a different function you should override this method to return the
     * value defined in the web.xml file.
     *
     * @return a string defining the event extension handled by this servlet
     */
    public String getEventExtension() {
        return ".event";
    }
    
    /**
     * <p>Handle the default HttpRequest. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefaultExt(req, resp, null);
    }

    /**
     * <p>Handle the default HttpRequest with the ability to add the provided
     * object into the context. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param contextObj object to put into the context
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefaultExt(HttpServletRequest req, HttpServletResponse resp, Object externalContextObj) throws ServletException, IOException {
        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) logger.info("Handling incoming HTTP request in "+this);

        BaseEvent event = null;
        
        //start by getting the event broker and pool. We get local copies so
        //that if these constant vaules change while we're mid-method, it's not a problem
        EventBroker eventBroker = getEventBroker();
        EventPool eventPool = getEventPool();
        ViewCapabilities vc = new ViewCapabilities(req, resp);    
        DefaultEventContext context = null;
        boolean useEventPool = USE_EVENT_POOLING;
        boolean eventCameFromPool = false;
        boolean allowEventAliasing = true;    //just do it always
        String target = null;
        
        try {
            //set up the Session ObjectRepository
            ObjectRepository.setupSessionRepository(req);                                  //csc_022101.1
            
            //figure out the name of the event handler being requested
            target = req.getServletPath();
            if (logger.isDebugEnabled()) {
                logger.debug("Incoming URI:"+req.getRequestURI());
                logger.debug("ServletPath:"+target);
                logger.debug("HTTP Headers...");        //csc_052803.1
                ServletUtil.showHeader(req, logger);    //csc_052803.1
            }

            // cater for Enhydra's different behaviour if in a PO
            if (null==target || target.length()==0) {
                target = req.getRequestURI();
                if (logger.isDebugEnabled()) logger.debug("Setting target to request URI: " + target);
            }

            //..trim any path info (based on '/')
            if (target!=null) {
                int spos = target.lastIndexOf("/");
                if (spos>-1) target = target.substring(spos+1);
            }
            //..trim any path info (based on '\')
            if (target!=null) {
                int spos = target.lastIndexOf("\\");
                if (spos>-1) target = target.substring(spos+1);
            }

            //now get the actual name of the event
            String ext = eventBroker.getEventExtension();
            String eventName = target;
            if (eventName.endsWith(ext)) {
                eventName = target.substring(0, target.length()-ext.length());
                if (logger.isDebugEnabled()) logger.debug("Target Event:"+eventName);
            }

            //now get the actual event name            
            if (allowEventAliasing) try {
                String fullEventName = eventBroker.matchEventClass(eventName);
                if (fullEventName!=null) {
                    eventName = fullEventName;
                    if (logger.isDebugEnabled()) logger.debug("Fully qualified Event:"+eventName);
                }
            } catch (InvalidClassException e) {}

            //now instantiate an Event (here's where it would make sense to
            //use an Event pool)
            if (logger.isDebugEnabled()) logger.debug("Creating Event");
            try {
                //get the class name for the event
 //csc_060903_3                Class cl = Thread.currentThread().getContextClassLoader().loadClass(eventName);
                Class cl = Classes.getClass(eventName); //csc_060903_3
                
                //get it from the pool if appropriate
                if (useEventPool) try {
                    if (logger.isDebugEnabled()) logger.debug("Checking out event from event pool");
                    event = eventPool.checkoutEvent(cl);
                    eventCameFromPool = true;
                } catch (NoAvailableEventsException e) {
                    if (logger.isDebugEnabled()) logger.debug("No available events in pool");
                }
                
                //if that didn't work (or we're not getting it from the
                //pool in the first place) create it manually
                if (event==null) {
                    if (logger.isDebugEnabled()) logger.debug("Creating a new event from scratch");
                    event = (BaseEvent) cl.newInstance();    //this requires that events being dispatched have a noargs constructor
                }
                
            } catch (Exception e) {
                //if we can't map the request to an event, we will just
                //dispatch a generic HttpRequestEvent
                logger.error("Class "+eventName+" not found...Creating a generic HttpRequestEvent");
                event = new HttpRequestEvent();
            }

            //make sure the incoming event implements HttpRequestEvent...if not,
            //consider it invalid (This is VERY important: it prevents people
            //from hacking the system by directly invoking events that were
            //not intended to be invoked directly from teh outside world)
            if (!(event instanceof HttpRequestEvent)) event = new HttpRequestEvent();

            //now set any special paramters
            event.setSource(this);
            event.setHandled(false);
            
            //now figure out if the event targets any specific listener IDs, and
            //if so add these ids to the event. This info will be used by the
            //dispatcher to send the event to the specified listeners, rather
            //doing a general dispatch to all parties interested in this class of
            //event.
            String[] ids = req.getParameterValues(BaseEvent.EVENT_ID);
            if (logger.isDebugEnabled()) logger.debug("Looking for event listener ids:"+ids);
            if (ids!=null) {
                for (int i=0, max=ids.length; i<max; i++) {
                    String id = ids[i];                
                    if (logger.isDebugEnabled()) logger.debug("Target EventID:"+id);
                    if (allowEventAliasing) try {
                        id = eventBroker.matchListenerID(ids[i]);
                        if (logger.isDebugEnabled()) logger.debug("Fully qualified EventID:"+id);
                    } catch (InvalidClassException e) {}
                    event.addListenerID(id);                
                }                
            }
            
            //finally, dispatch it to the EventBroker
            if (logger.isDebugEnabled()) logger.debug("Dispatching event:"+event+" to broker:"+eventBroker);
//csc_100501.1            DefaultDispatchQueue eventQueue = new DefaultDispatchQueue(true);
            DispatchQueue eventQueue = getNewDispatchQueueInstance();    //csc_100501.1
            HttpResponseEvent defaultResponseEvent = new HttpResponseEvent();
            defaultResponseEvent.setSource(event);
            eventQueue.addEvent(event);
//csc_100901.1            DefaultEventContext context = new DefaultEventContext(eventQueue, vc, config, req, resp, defaultResponseEvent);
            context = new DefaultEventContext(eventQueue, vc, this.getServletConfig(), req, resp, defaultResponseEvent);
            context.putState(TARGET_EVENT_NAME, eventName);
            if (null!=externalContextObj) {
                context.putState(EXTERNAL_CONTEXT_OBJ_NAME, externalContextObj);
            }
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            eventBroker.dispatchEvent(context);
            if (logger.isInfoEnabled()) emillis = System.currentTimeMillis();

            if (logger.isInfoEnabled()) logger.info("Dispatching complete! (handled in "+(emillis-smillis)+" of "+(emillis-bmillis)+" millis)");

        } catch (ClientSideRedirectException re) {
//csc_082302.2_start - moved up from DefaultEventDispatcher
            //save current context to session
            if (context!=null) {
                if (logger.isDebugEnabled()) logger.debug("Saving context");
                context.persistContext(re);
            }
//csc_082302.2_end


            //if we get a redirect exception, request the browser to redirect accordingly
            String url = URLRewriter.encodeRedirectURL(req, resp, re.getRedirectURL());
            url = ScriptDetector.prepareRedirectURL(url, vc);     //csc_102501.2
            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+url);
            resp.sendRedirect(url);    

        } catch (EventException e) {
            //if we get an EventException, handle it
            handleEventException(e, req, resp);

        } finally {
            //make sure we ALWAYS release the event
            if (event!=null && eventCameFromPool) {
                if (logger.isDebugEnabled()) logger.debug("Releasing event");
                eventPool.releaseEvent(event);
                if (logger.isDebugEnabled()) logger.debug("Released!");
            }
            
            //make sure we always clean up session/local repository stuff
            ObjectRepository.removeSessionRepository();                                     //csc_022101.1
            ObjectRepository.removeLocalRepository();                                       //csc_022101.1
        }
    }

    /**
     * <p>Handle an EventException. Basically, this is where we handle the
     * really bad, unexpected type of event exceptions. Generally, as you code,
     * if you want to interrupt the dispatch and fire a new event, you should
     * throw an InterruptDispatchException. Only throw EventExceptions in
     * truly exceptional circumstances.
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleEventException (EventException e, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //first see what the base exception is...if it's a servlet exception
        //or an IOException, rethrow it...
        Exception rootException = NestableException.getRootException(e);
        if (rootException instanceof ServletException) throw (ServletException) rootException;
        if (rootException instanceof IOException) throw (IOException) rootException;

        //if we get an EventException, log it and print the base exception
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        ExceptionUtil.logExceptionAsHTML(out, e, req);
        logger.warn("Unexpected event exception: ", e);
        if (rootException!=e) {
            logger.warn("Root Exception...", rootException);
        }
    }


    //-------------------- ApplicationGateway --------------------
    /**
     * <p>Specify event gateways. You can call this method with
     * as many different gateways as you like. When the Application
     * Gateway initializes, it will load them all. When the gateway
     * is destroyed, they will be deregistered.
     *
     * @param gateway an event gateway we'd like to have registered
     */
    public final void specifyEventGateways(EventGateway gateway) {
        if (gateways==null) gateways = new ArrayList();
        gateways.add(gateway);
    }
    
    /**
     * <p>Provide a reference to the event broker.
     *
     * @return a reference to the master event broker
     */
    public final EventBroker getEventBroker() {
        if (masterEventBroker==null) {
            masterEventBroker = getNewEventBrokerInstance();
            
            //this ensures that the App-gateway can successfully route
            //generic ActionEvents (if we ever add other "well-known"
            //generic events we should pre-register them here as well
            //so that they can be used without any special registration
            //process)
            try {masterEventBroker.addEventAlias(ActionEvent.class);}
            catch (InvalidClassException e) {}
        }
        return masterEventBroker;
    }

    /**
     * <p>Provide a reference to the event pool. May return
     * null if we're not using event pooling.
     *
     * @return a reference to the master event pool
     */
    public final EventPool getEventPool() {
//csc_060903_1        if (USE_EVENT_POOLING && masterEventPool==null) masterEventPool = getNewEventPoolInstance();
        if (USE_EVENT_POOLING && masterEventPool==null) masterEventPool = (EventPool) Classes.newInstance(A_Classes.DEFAULT_EVENT_POOL);    //csc_060903_1
        if (!USE_EVENT_POOLING) masterEventPool = null;
        return masterEventPool;
    }
    

    //dbr_021602 - added
    private org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper wrap(HttpServletRequest req) {
        if (REQUEST_WRAPPER!=null) {
            return REQUEST_WRAPPER.wrap(req);
        }
        return new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(req);
    }




    
    

    
    //-------------------- EventGateway --------------------------
    /**
     * <p>Set the parent gateway. Null indicates its the root.
     * By definition, an ApplicationGateway is always the root.
     *
     * @param eg the parent event gateway for this gateway
     */
    public final void setParent(EventGateway eg) {
        eventGateway.setParent(null);
    }
    
    /**
     * <p>Get the parent gateway. Returns null if it's the root.
     *
     * @return the parent event gateway
     */
    public final EventGateway getParent() {
        return eventGateway.getParent();
    }
    
    /**
     * <p>Add an event gateway to this one
     *
     * @param eg the event gateway to be added
     */
    public final void add(EventGateway eg) {
        eventGateway.add(eg);
        eg.setParent(this);    //do this so the parent points at the ApplicationGateway, not the EventGateway we are using to actually store the values
    }
    
    /**
     * <p>Remove an event gateway from this one
     *
     * @param eg the event gateway to be removed
     */
    public final void remove(EventGateway eg) {
        eventGateway.remove(eg);
    }
    
    /**
     * Get a list of child gateways. The list returned is a copy of the
     * underlying child gateway list.
     *
     * @return a list of child gateways
     */
    public List getChildren() {
        return eventGateway.getChildren();
    }
    
    /**
     * <p>Ask all interested parties to register with
     * the EventBroker
     *
     * @param eb the event broker to register with
     */
    public final void register(EventBroker eb) {
        eventGateway.register(eb);
    }

    /**
     * <p>Ask all interested parties to de-register with
     * the EventBroker
     *
     * @param eb the event broker to de-register with
     */
    public final void deregister(EventBroker eb) {
        eventGateway.deregister(eb);
    }

    /**
     * Register any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register for local events
     */
    public final void registerLocalEventInterests(EventBroker eb) {
        eventGateway.registerLocalEventInterests(eb);
    }

    /**
     * Deregister any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        de-register for local events
     */
    public final void deregisterLocalEventInterests(EventBroker eb) {
        eventGateway.deregisterLocalEventInterests(eb);
    }

    /**
     * Register any local event aliases in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register aliases for local events
     */
    public final void registerLocalEventAliases(EventBroker eb) {
        eventGateway.registerLocalEventAliases(eb);
    }

    /**
     * Rather than overriding the registerLocalEventInterests 
     * method, you can just invoke this method instead for each 
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when 
     * appropriate (so you don't have to worry about it). Notice that
     * this method registers just the listener id (not for a specific
     * class of event).
     *
     * The only real reason for using the registerLocalEventInterests 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     * @param event the class of events we are interested in
     */
    public final void specifyLocalEventInterests(ListenerFactory factory) {
        eventGateway.specifyLocalEventInterests(factory);
    }

    /**
     * Rather than overriding the registerLocalEventInterests 
     * method, you can just invoke this method instead for each 
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when 
     * appropriate (so you don't have to worry about it). 
     *
     * The only real reason for using the registerLocalEventInterests 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     * @param event the class of events we are interested in
     */
    public final void specifyLocalEventInterests(ListenerFactory factory, Class event) {
        eventGateway.specifyLocalEventInterests(factory, event);
    }

    /**
     * Rather than overriding the registerLocalEventAliases 
     * method, you can just invoke this method instead for type
     * of event you want to manually alias.
     *
     * The only real reason for using the registerLocalEventAliases 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param event the class of events we are interested in registering 
     *        aliases for
     */
    public final void specifyLocalEventAliases(Class event) {
        eventGateway.specifyLocalEventAliases(event);
    }






    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
//csc_102201.1_start
//        handleDefault(new org.enhydra.barracuda.core.helper.servlet.HttpServletRequestWrapper(wrap(req)), resp);
        boolean handled = ScriptDetector.checkClientReq(req, resp);
        //dbr_021602_start.2        
        if (!handled) handleDefault(wrap(req), resp);
        //dbr_021602_end.2        
//csc_102201.1_end
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrap(req), resp);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrap(req), resp);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrap(req), resp);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrap(req), resp);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        handleDefault(wrap(req), res);
    }






    //-------------------- Servlet -------------------------------
    /**
     * <p>Here's where we initialize the servlet.
     *
     * @param config the servlet config object this servlet is
     *        being initialized with
     */
    public void init() throws ServletException {

        try {
            logger.info("initializing servlet");


//csc_060903_3_start - why are we even doing this section??? This seems redundant to me, like its
//something that should be happening via the ObjectRepositoryAssembler, NOT ApplicationGateway
//parameters...Jake seems to think so too, so I'm commenting it out for now
/*
            //first and foremost, assign any debug value constants
            logger.debug("checking for DebugValues");
            Enumeration enum = this.getServletConfig().getInitParameterNames();
            while (enum.hasMoreElements()) {
                String param = enum.nextElement().toString();
                if (param.startsWith(DEBUG_VALUE)) {
                    String value = this.getServletConfig().getInitParameter(param);
                    logger.debug(param+" = "+value);
                
                    //split it into parts
                    try  {
                        int pos1 = value.lastIndexOf(".");
                        int pos2 = value.indexOf("=", pos1);
                        String clname = value.substring(0,pos1);
                        String var = value.substring(pos1+1, pos2);
                        String val = value.substring(pos2+1);
                        logger.debug("Class:"+clname+" Variable:"+var+" Value:"+val);

                        //figure out what value we're trying to set
//csc_060903_3                        Class cl = Thread.currentThread().getContextClassLoader().loadClass(clname);
                        Class cl = Classes.getClass(clname);    //csc_060903_3

                        Field field = cl.getDeclaredField(var);
                        Object origval = field.get(null);
                        Class fieldType = field.getType();
                        Object targetval = null;
                        //...integer
                        if (fieldType==Integer.class || fieldType==int.class) {    
                            targetval = new Integer(val);
                        //...boolean
                        } else if (fieldType==Boolean.class || fieldType==boolean.class) {
                            targetval = new Boolean(val);                        
                        //...float
                        } else if (fieldType==Float.class || fieldType==float.class) {    
                            targetval = new Float(val);
                        //...double
                        } else if (fieldType==Double.class || fieldType==double.class) {    
                            targetval = new Double(val);
                        //...long
                        } else if (fieldType==Long.class || fieldType==long.class) {    
                            targetval = new Long(val);
                        //...short
                        } else if (fieldType==Short.class || fieldType==short.class) {    
                            targetval = new Short(val);
                        //...String (def)
                        } else {
                            targetval = val;
                        }
                        
                        //actually try setting it
                        field.set(null, targetval);
                        Object newval = field.get(null);
                        logger.debug("set "+clname+"."+var+" from "+origval+" to "+newval);

                    } catch (Exception e) {                
                        logger.error("err "+e+" setting value:"+value);
                        continue;                    
                    }
                }
            }
*/
//csc_060903_3_end

//csc_060903_2_start - move down below so that EventPool could be scripted via the application assembler
            //start by getting a reference to the EventBroker. Also request the
            //EventPool, just to make sure they're both instantiated
//            logger.debug("creating EventBroker");
//            EventBroker eb = this.getEventBroker();
//            getEventPool();
//csc_060903_2_end

            //perform any automated assembly if necessary
            String assemblerName = this.getServletConfig().getInitParameter(APPLICATION_ASSEMBLER);
            String descriptor = this.getServletConfig().getInitParameter(ASSEMBLY_DESCRIPTOR);
            if (assemblerName!=null && descriptor!=null) try {
                logger.debug("performing automated assembly");
                String parser = this.getServletConfig().getInitParameter(SAX_PARSER);
//csc_060903_1                ApplicationAssembler assembler = (ApplicationAssembler) Thread.currentThread().getContextClassLoader().loadClass(assemblerName).newInstance();
                ApplicationAssembler assembler = (ApplicationAssembler) Classes.newInstance(assemblerName);  //csc_060903_1
                if (parser!=null) assembler.assemble(this, this.getServletConfig(), descriptor, parser);
                else assembler.assemble(this, this.getServletConfig(), descriptor);
                logger.debug("automated assembly complete!");
            } catch (Exception e) {
                throw new RuntimeException ("Error invoking assembler:"+e);
            }

//csc_060903_2_start
            //start by getting a reference to the EventBroker. Also request the
            //EventPool, just to make sure they're both instantiated. We do this here 
            //primarily so that if there is a problem instantiating these, we find 
            //out about it immediately on startup, rather than waiting until a request
            //actually comes through.
            logger.debug("creating EventBroker");
            EventBroker eb = this.getEventBroker();
            getEventPool();
//csc_060903_2_end

            //allow for local initialization
            logger.debug("initialize local");
            initializeLocal();

            //handle any gateways which were specified
            if (gateways!=null) {
                logger.debug("handle any local gateways");
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    logger.debug("adding specified event gateway:"+gateway);
                    this.add(gateway);
                }
            }

            //now register the event gateway (note that we have to set the value to
            //false when registering to ensure that all possible combinations get
            //registered. If we don't do this we run into problems later)
            logger.debug("invoke register");
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            register(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;
            
            //show the app structure
/*
//csc_060903_3 - removed because it is calling methods that are not part of the EventGateway interface (and its not really used anyway)            
            if (SHOW_APP_STRUCTURE) {
                logger.debug("Printing Stack Trace...");
                eventGateway.printStackTrace(0, System.out);
            }
*/        
            if (logger.isInfoEnabled()) logger.info("Initialization Complete!");
        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err initializing servlet:", e);
            throw e;
        }
    }
        
    /**
     * <p>Here's where we destroy the servlet.
     */
    public void destroy() {
        try {
            logger.info("destroying servlet");

            //start by getting a reference to the EventBroker
            EventBroker eb = this.getEventBroker();

            //now deregister the event gateway
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            deregister(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;
            
            //finally allow for any local destruction
            destroyLocal();
            
            //remove any gateways which were specified
            if (gateways!=null) {
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    this.remove(gateway);
                }
            }

            //lb_032801_start - Patch submitted by Larry Brasfield
            //[larry.brasfield@theplatform.com] so JVM can exit once
            //the servlet's destroy() method is called.  The problem
            //is that the event pool cleanup thread will not stop. The
            //following code corrects this. You can find related changes
            //by searching for lb_032801
            //
            //Allow finalizer to get rid of this pool object.
            //That way, a shutdown can actually be effected.
            if (masterEventPool!=null) {
                masterEventPool.shutdown();
                masterEventPool = null;
            }
            masterEventBroker = null;
            System.gc();
            System.runFinalization();
            //lb_032801_end
                    
        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err destroying servlet:", e);
            throw e;
        }
    }
    
    
}
