/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DefaultBaseEvent.java,v 1.12 2003/05/28 16:07:09 christianc Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.plankton.data.*;

/**
 * <p>This is the default implementation for the BaseEvent
 * interface. It acts as the base class in the event heirarchy.
 *
 * <p>While events are capable of carrying state information,
 * generally you want to keep them as lite as possible, storying
 * any state either in the dispatch queue or the user's session.
 *
 * <p>NOTE: As general good practice, you should try and provide
 * a no-args public constructor for all event classes. Anything
 * that implements Polymorphic or Exceptional MUST have a noargs
 * constructor, or the system will not be able to instantiate it.
 */
public abstract class DefaultBaseEvent implements BaseEvent {
    //constants
    public static boolean USE_ID_ALIASES = true;            //use id aliases (default setting)?

    //private vars
    protected Object source = null;                         //the source of the event
    protected EventObject sourceEvent = null;               //the source event for this event (if applicable)
    protected String ext = ".event";                        //default event extension
    protected boolean handled = false;                      //has this event been handled
    protected List idList = null;                           //list of IDs this event is specifically targeted towards
    protected long timestamp = -1;                          //last time this event was touched
    protected boolean useIDAliases = USE_ID_ALIASES;
    protected StateMap statemap = new DefaultStateMap();    //private property map
    protected Map params = null;                            //private parameter map //jbh_112202.1

    /**
     * Default noargs constructor
     */
    public DefaultBaseEvent() {super();}

    /**
     * Public constructor. Automatically sets the source parameter.
     * If you do not use this method you should manually set the
     * source before dispatching the event.    
     */
    public DefaultBaseEvent(Object source) {
        setSource(source);
    }

    /**
     * set the source for an event
     *
     * @param isource the source for this event
     */
    public void setSource(Object isource) {
        source = isource;
    }
    
    /**
     * get the event source (may be null)
     *
     * @return the source for this event
     */
    public Object getSource() {
        return source;
    }
    
    /**
     * get the root event source (may be null)
     *
     * @return the root source for this event
     */
    public BaseEvent getRootEvent() {
        if (source!=null && source instanceof BaseEvent) return ((BaseEvent) source).getRootEvent();
        else return this;
    }
    
    /**
     * set the event extension
     *
     * @param iext the target event extension
     */    
    public void setEventExtension(String iext) {
        ext = iext;
    }
    
    /**
     * get the event extension
     *
     * @return the target event extension
     */    
    public String getEventExtension() {
        return ext;
    }

    /**
     * determine whether or not we are using ID aliasing. This
     * can be used to override the default USE_ID_ALIASES
     * setting.
     *
     * @param val true if we want to use ID aliasing
     */
    public void setUseIDAliases(boolean val) {
        useIDAliases = val;
    }
    
    /**
     * see whether or not we are using ID aliasing
     *
     * @return true if we want to use ID aliasing
     */
    public boolean useIDAliases() {
        return useIDAliases;
    }
    
    /**
     * mark the event as handled/unhandled
     *
     * @param true if the event is handled
     */    
    public void setHandled(boolean val) {
        handled = val;
    }
    
    /**
     * get the handled status for the event
     *
     * @return true if the event is handled
     */    
    public boolean isHandled() {
        return handled;
    }

    /**
     * Add a specific listener id this event should be delivered to.
     * Events can be targeted to more than one ID.
     *
     * @param id the Listener ID the event should target
     */        
    public void addListenerID(String id) {
        if (idList==null) idList = new ArrayList();
        idList.add(id);
    }

    /**
     * Get the list of id's this event is specifically targeted for.
     * May return null if there are none.
     *
     * @return a List of ID's this event is specifically targeting
     */        
    public List getListenerIDs() {
        return idList;    
    }

    /**
     * Get the ID that identifies this event. This will typically be the
     * class name.
     *
     * @return a string that uniquely identifies this event
     */
    public String getEventID() {
        String idStr = this.getClass().getName();
        if (useIDAliases()) {
            int spos = idStr.lastIndexOf('.');
            if (spos>-1) idStr = idStr.substring(spos+1);
        }
        return idStr;
    }
    
    // RTL20010719
    /**
     * Get the id + extension of this event.
     *
     * @return the id and extension of this event
     */
    public String getEventIDWithExtension() {
        return (this.getEventID()+this.getEventExtension());
    }
    // end RTL20010719

    /**
     * Get the timestamp
     *
     * @return the last time this event was touched
     */
    public long getTimestamp() {
        return timestamp;
    }
    
    /**
     * Update the timestamp on the event
     */
    public void touch() {
        timestamp = System.currentTimeMillis();
    }

    /**
     * Reset the event to it's default state
     */    
    public void reset() {
        source = null;
        sourceEvent = null;
        ext = ".event";
        handled = false;
        idList = null;
        statemap = new DefaultStateMap();
        touch();
    }
    
    /**
     * Find the original event in target event's event chain (if it exists)
     *
     * @param e the target event
     * @return the original BaseEvent that caused the target event
     */
    public synchronized static BaseEvent getOriginalEvent(BaseEvent e) {
        Object source = e.getSource();
        if (source!=null && source instanceof BaseEvent) return getOriginalEvent((BaseEvent) source);
        else return e;
    }
    
    /**
     * Events are generally considered equal if they are of the same class.
     * If you don't want this to be the case for some events (ie. if you
     * want to get every instance of an event through your dispatcher) then
     * you should override this method.
     *
     * @param o the object we are checking against for equality
     * @return true if the objects are equal
     */    
    public boolean equals(Object o) {
        if (o==null) return false;
        return (o==this || o.getClass().equals(this.getClass()));
    }
    
    /**
     * clone the event
     *
     * @return a copy of this event
     * @throws CloneNotSupportedException
     */
    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }
    
    /**
     * Get a class ID for a given class. The USE_ID_ALIASES
     * parameter is taken into account when generating the ID.
     *
     * @param cl the target class
     * @return the class ID for the target class
     */
    public static String getClassID(Class cl) {    
        //eliminate the obvious
        if (cl==null) return null;

        String idStr = cl.getName();
        if (USE_ID_ALIASES) {
            int spos = idStr.lastIndexOf('.');
            if (spos>-1) idStr = idStr.substring(spos+1);
        }
        return idStr;
    }
    
    /**
     * get the RootEvent that caused this event (if any). Will look at
     * the source object to see if it happens to be an instance of a
     * BaseEvent, and will recursively work deeper until it finds
     * the event which caused it all
     *
     * @param be a BaseEvent for which we wish to find the root event
     * @return the root BaseEvent in an event chain (may return null)
     */
    public synchronized static BaseEvent getRootEvent (BaseEvent be) {
        Object source = be.getSource();
        if (source!=null && source instanceof BaseEvent) return getRootEvent ((BaseEvent) source);
        return be;
    }
    
    //-------------------- StateMap ------------------------------
    /**
     * set a property in this StateMap
     *
     * @param key the state key object
     * @param val the state value object
     */    
    public void putState(Object key, Object val) {
        statemap.putState(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the state key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return statemap.getState(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return statemap.removeState(key);
    }
    
    /**
     * get a list of the keys for this StateMap
     *
     * @return a list the keys for this StateMap
     */    
    public List getStateKeys() {
        return statemap.getStateKeys();
    }
    
    /**
     * get a copy of the underlying Map
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        return statemap.getStateValues();    
    }
    
    //csc_052803_2 - added
    /**
     * clear all state information
     */
    public void clearState() {
        statemap.clearState();
    }
    

    
    //jbh_112202.1_start
    //-------------------- Params --------------------------------
    /**
     * Set any associated params
     */
    public void setParam(String key, String val) {
        if (params==null) params = new HashMap(10);
        params.put(key, val);
    }

    /**
     * Set any associated params
     */
    public void setParam(String key, String[] val) {
        if (params==null) params = new HashMap(10);
        params.put(key, val);
    }

    /**
     * Get any associated params
     */
    public Map getParams() {
        return params;
    }
    //jbh_112202.1_end
    
}
