/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventBroker.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

/**
 * This interface defines the methods needed to implement an EventBroker
 */
public interface EventBroker {

    /**
     * Return the event extension handled by this event broker
     *
     * @return the event extension associated with this broker
     */
    public String getEventExtension();

    /**
     * register a listener id, so that events addressed to a 
     * specific listener can be delivered
     *
     * @param factory the listener factory to be added
     */    
    public void addEventListener(ListenerFactory factory);

    /**
     * add an event listener for a particular class of an event.
     * If the class referenced is not an instance of BaseEvent,
     * a ClassNotEventException will be thrown. Note that this 
     * method also registers the listener in the idMap as well.
     *
     * @param factory the listener factory to be added
     * @param event the specific class of event for which the factory is listening
     * @throws InvalidClassException if the event class does not implement BaseEvent
     */    
    public void addEventListener(ListenerFactory factory, Class event) throws InvalidClassException;

    /**
     * remove a listener from general availability
     *
     * @param factory the listener factory to be removed
     */
    public void removeEventListener(ListenerFactory factory);

    /**
     * remove an event listener for specific types of events
     * If the class referenced is not an instance of BaseEvent,
     * a InvalidClassException will be thrown. Note that this 
     * method effectively deregisters the listener from the 
     * idMap as well.
     *
     * @param factory the listener factory to be removed
     * @param event the specific class of event for which the factory is listening
     * @throws InvalidClassException if the event class does not implement BaseEvent
     */
    public void removeEventListener(ListenerFactory factory, Class event) throws InvalidClassException;

    /**
     * remove all references to an event listener, both for id and
     * for any event classes it has registered an interest in.
     *
     * @param factory the listener factory to be removed
     */
    public void purgeEventListener(ListenerFactory factory);

    /**
     * Manually register aliases for a given event (the aliases
     * will be determined automatically based on the class name
     * and the event ID)
     *
     * @param event the specific class of event we'd like to alias
     * @throws InvalidClassException if the event class does not implement BaseEvent
     */
    public void addEventAlias(Class event) throws InvalidClassException;
 
    /**
     * Manually add an alias for a given event. Note that
     * the alias parameter will be converted into all possible 
     * aliases based on '.' delimiters.
     *
     * @param event the specific class of event we'd like to alias
     * @param alias the alias for this event
     * @throws InvalidClassException if the event class does not implement BaseEvent
     */
    public void addEventAlias(Class event, String alias) throws InvalidClassException;

    /**
     * Get a specific listener based on listener ID
     *
     * @param id the listener id we're looking for
     * @return the ListenerFactory that matches that id
     */
    public ListenerFactory getEventListener(Object id);

    /**
     * Get a List of listeners for a type of event. Returns a copy 
     * of the broker's internal list, so you can do what you want 
     * with it. If the class referenced is not an instance of BaseEvent,
     * a InvalidClassException will be thrown. 
     *
     * @param event the event class we are looking for
     * @return a List of listeners that are interested in this class of event
     * @throws InvalidClassException if the event class does not implement BaseEvent
     */
    public List getEventListeners(Class event) throws InvalidClassException;

    /**
     * Given a partial event class name, return the fully qualified class
     * name if it's possible to determine. If it is unknown, throw and
     * InvalidClassException. This method is primarily used by the 
     * ApplicationGateway to support event aliasing.
     *
     * @param eventStr the event name alias
     * @return the fully qualified event class name
     * @throws InvalidClassException if the eventStr cannot be unambiguously 
     *      matched to a class name
     */
    public String matchEventClass(String eventStr) throws InvalidClassException;

    /**
     * Given a partial id name, return the fully qualified listener
     * ID if it's possible to determine. If it is unknown, throw and
     * InvalidClassException. This method is primarily used by the 
     * ApplicationGateway to support id aliasing.
     *
     * @param idStr the id name alias
     * @return the fully qualified listener id name
     * @throws InvalidClassException if the idStr cannot be unambiguously 
     *      matched to a listener id name
     */
    public String matchListenerID(String idStr) throws InvalidClassException;


    /**
     * <p>Dispatch a queue of events. Generally, the queue will only
     * contain one event, however, if you ever need to dispatch
     * multiple events at once, the broker can handle it. All the real 
     * dispatching work is carried out by the underlying event
     * dispatcher.
     *
     * <p>The event queue you pass in should contain several pieces of 
     * state information:
     *
     * <ul>
     *    <li>DefaultEventDispatcher.DEFAULT_RESPONSE_EVENT (BaseEvent)</li>
     *    <li>DefaultBaseEventListener.HTTP_SERVLET_REQUEST (HttpServletRequest) - if dispatching from a servlet</li>
     *    <li>DefaultBaseEventListener.HTTP_SERVLET_RESPONSE (HttpServletResponse) - if dispatching from a servlet</li>
     * </ul>
     *
     * @param eventQueue a queue of one or more events to be dispatched
     * @throws EventException
     */
    public void dispatchEvent(EventContext context) throws EventException;

    /** @link dependency */
    /*#EventDispatcher lnkEventDispatcher;*/
}
