/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Stephen Peterson, Jacob Kjome
 *
 * $Id: EventConnectorFactory.java,v 1.7 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.event.*;

/**
 * <p> This class associates a specific event handler class with an event by 
 * extending DefaultListenerFactory and making implementation details in the 
 * gateways much simpler... Without this class, each instance of a listener 
 * factory must create a separate anonymous class and provide the
 * implementation for getInstance() and getListenerID().
 *
 * <p>With this class, the association of an event to a handler is a reduced
 * to a single line:
 * 
 * <p><code>specifyLocalEventInterests(new EventConnectorFactory(Handler.class), Event.class);</code>
 * 
 * <p> IMPORTANT NOTE: When using this class, the passed-in event handler must 
 * either be a non-inner class or an inner class which is declared public static. 
 * Otherwise, when the getInstance() method attempts to instantiate the handler a 
 * <code>java.lang.InstantiationException</code> will get thrown which will show up 
 * as a <code>java.lang.NullPointerException</code>. This is because getInstance() 
 * returns null if it fails to instantiate the handler. This won't be caught 
 * at compile time, so be careful!
 *
 * @author  Stephen Peterson <stephen_peterson@agilent.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0.1 (2002-02-15)
 */
public class EventConnectorFactory extends DefaultListenerFactory {

    protected static final Logger logger = Logger.getLogger(EventConnectorFactory.class.getName());
    protected Class eventHandlerClass = null;

    /**
     * Constructor
     *
     * @param the handler class to associate with an event
     */
    public EventConnectorFactory(Class aClass) {
        logger.debug("Creating new EventConnectorFactory --> "+aClass);
        
        //sanity check & assignment
        eventHandlerClass = aClass;
        if (eventHandlerClass==null) {
            logger.fatal("Event handler class cannot be null!", new IllegalAccessException());
        }
        
        //now go ahead and invoke getInstance(), just to make sure its going
        //to work later. This will effectively catch errors immediately at 
        //startup, rather than waiting until someone stumbles into the problem 
        //while using the application
        getInstance();
    }

    /**
     * Get an instance of the underlying BaseEventListener
     *
     * @return get an instance of the BaseEventListener
     */
    public BaseEventListener getInstance() {
        if (logger.isDebugEnabled()) logger.debug("Creating instance of: " + eventHandlerClass.getName());

        //create an instance of the class
        BaseEventListener bel = null;
        Exception e = null;
        try {
            bel = (BaseEventListener) eventHandlerClass.newInstance();
        } catch (IllegalAccessException iae) {
            logger.fatal("Illegal Access Exception!", iae);
        } catch (InstantiationException ie) {
            logger.fatal("Error instantiating "+eventHandlerClass.getName()+"; if you defined it as an inner class, make sure its declared public static", ie);
        } catch (ClassCastException cce) {
            logger.fatal("Error casting "+eventHandlerClass.getName()+" to BaseEventListner. This class or its superclass must implement the BaseEventListener interface", cce);
        }

        //return the event listener
        return bel;
    }

    /**
     * Get the Listener ID associated with this class of listener. This will
     * generally either be the class name of the listener that the factory
     * creates
     *
     * @return the listener ID that describes this factory
     */
    public String getListenerID() {
        return getID(eventHandlerClass);
    }

}
