/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DefaultFormElement.java,v 1.19 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.plankton.data.DefaultStateMap;
import org.enhydra.barracuda.plankton.data.StateMap;
import org.apache.log4j.*;
import java.math.BigDecimal;

/**
 * <p>A FormElement defines how an element in a FormMap should
 * be mapped to a first class java object. There are several
 * key pieces of information required:
 *
 * <ul>
 *         <li>key - used to retrieve the target value from the incoming
 *            data source (either a ServletRequest or a StateMap)</li>
 *         <li>name - a displayable name for this element (defaults to key)</li>
 *         <li>type - the type of data object the incoming value should be
 *            mapped to, as defined by the FormType class (String, Boolean,
 *            Integer, Date, Long, Short, Double, Float)</li>
 *         <li>default val - the default val to be used if the key value is
 *            null in the incoming data source (may be null)</li>
 *         <li>validator - any validators associated with this particular
 *            form elmenent (may be null)</li>
 *         <li>allow multiples - the servlet interface allows multiple params
 *            with the same key. This value indicates whether or not that is
 *            acceptable for a key. If so, then when multiple values are
 *            encountered they element will actually contain a List of values
 *            of the proper type</li>
 * </ul>
 *
 * <p>Once we have this information, we basically have enough data to map
 * a String value in the data source to a first class Java object in the
 * element.
 *
 * <p>Note that the element keeps track of the original value as well, so
 * that if for some reason you ever need to get to it that information is
 * available.
 *
 * <p>Also not that we provide convenience methods which automatically cast
 * for you (Note however: this type of thing is not type safe, meaning that
 * you can get a ClassCastException if you do a getXXXVal() when the
 * underlying object is not of type XXX.
 */
public class DefaultFormElement implements FormElement {

    //public constants
    protected static final Logger logger = Logger.getLogger(DefaultFormElement.class.getName());

    //non-public vars
    protected String key = null;
    protected String name = null;
    protected FormType type = null;
    protected Object defaultVal = null;
    protected ParseException pe = null;
    protected FormValidator validator = null;
    protected boolean allowMultiples = false;
    protected Object origVal = null;
    protected Object val = null;

    /**
     * Public noargs constructor. Form key defaults to null.
     */
    public DefaultFormElement() {
        this(null);
    }
    
    /**
     * Public constructor. Form type defaults to FormType.STRING
     *
     * @param key the key name in the data source
     */
    public DefaultFormElement(String ikey) {
        this(ikey, FormType.STRING);
    }

    /**
     * Public constructor. Default value defaults to Null.
     *
     * @param key the key name in the data source
     * @param type the FormType for the element
     */
    public DefaultFormElement(String ikey, FormType itype) {
        this(ikey, itype, null);
    }

    /**
     * Public constructor. Validator defaults to null.
     *
     * @param key the key name in the data source
     * @param type the FormType for the element
     * @param defaultVal the default value to be used if the key is
     *        not found in the data source or the value for the key is null
     */
    public DefaultFormElement(String ikey, FormType itype, Object idefaultVal) {
        this(ikey, itype, idefaultVal, null);
    }

    /**
     * Public constructor. AllowMultiples defaults to false.
     *
     * @param key the key name in the data source
     * @param type the FormType for the element
     * @param defaultVal the default value to be used if the key is
     *        not found in the data source or the value for the key is null
     * @param validator the FormValidator associated with this element
     */
    public DefaultFormElement(String ikey, FormType itype, Object idefaultVal, FormValidator ivalidator) {
        this(ikey, itype, idefaultVal, ivalidator, false);
    }
    
    /**
     * Public constructor, Name defaults to Key
     *
     * @param key the key name in the data source
     * @param type the FormType for the element
     * @param defaultVal the default value to be used if the key is
     *        not found in the data source or the value for the key is null
     * @param validator the FormValidator associated with this element
     * @param allowMultiples true if there may be multiple values in the datasource
     *        for this particular key name
     */
    public DefaultFormElement(String ikey, FormType itype, Object idefaultVal, FormValidator ivalidator, boolean iallowMultiples) {
//csc_100901.3        this(ikey, ikey, itype, idefaultVal, ivalidator, false);
        this(ikey, ikey, itype, idefaultVal, ivalidator, iallowMultiples);
    }

    /**
     * Public constructor
     *
     * @param key the key name in the data source
     * @param name the name for the element
     * @param type the FormType for the element
     * @param defaultVal the default value to be used if the key is
     *        not found in the data source or the value for the key is null
     * @param validator the FormValidator associated with this element
     * @param allowMultiples true if there may be multiple values in the datasource
     *        for this particular key name
     */
    public DefaultFormElement(String ikey, String iname, FormType itype, Object idefaultVal, FormValidator ivalidator, boolean iallowMultiples) {
        setKey(ikey);
        setName(iname);
        setType(itype);
        setDefaultVal(idefaultVal);
        setValidator(ivalidator);
        setAllowMultiples(iallowMultiples);
    }

    /**
     * Set the key value for this form element
     *
     * @param key the key value for this form element
     */
    public void setKey(String ikey) {
        key = ikey;
    }
    
    /**
     * Get the key value for this form element
     *
     * @return the key for this form element
     */
    public String getKey() {
        return key;
    }

    /**
     * Set the name of this form element
     *
     * @param iname the name of this form element
     */
    public void setName(String iname) {
        name = iname;
    }
    
    /**
     * Get the name of this form element
     *
     * @return the name of this form element
     */
    public String getName() {
        return name;
    }

    /**
     * Set the FormType for this form element
     *
     * @param type the FormType for this form element
     */
    public void setType(FormType itype) {
        if (itype==null) itype = FormType.STRING;
        type = itype;
    }
    
    /**
     * Get the FormType for this for element
     *
     * @return the FormType for this form element
     */
    public FormType getType() {
        return type;
    }

    /**
     * Set the default value for this form element
     *
     * @param defaultVal the FormType for this form element
     */
    public void setDefaultVal(Object idefaultVal) {
        defaultVal = idefaultVal;
    }
    
    /**
     * Get the default value for this form element
     *
     * @return the default value for this form element (may be null)
     */
    public Object getDefaultVal() {
        return defaultVal;
    }

    /**
     * Set whether or not this element allows multiple values
     *
     * @param val true if the element allows multiples
     */
    public void setAllowMultiples(boolean val) {
        allowMultiples = val;
    }
    
    /**
     * Does this element allow multiple values
     *
     * @return true if this element allows multiple values
     */
    public boolean allowMultiples() {
        return allowMultiples;
    }

    /**
     * Set the FormValidator for this form element
     *
     * @param validator the FormValidator for this form element
     */
    public void setValidator(FormValidator ivalidator) {
        validator = ivalidator;
    }
    
    /**
     * Get the default FormValidator for this form element
     *
     * @return the validator for this form element (may be null)
     */
    public FormValidator getValidator() {
        return validator;
    }

    /**
     * Set the original value for this element
     *
     * @param origVal the original value
     */
    public void setOrigVal(Object iorigVal) {
        origVal = iorigVal;
    }

    /**
     * Get the original value for this element
     *
     * @return the original value for this form element (may be null)
     */
    public Object getOrigVal() {
        return origVal;
    }

    /**
     * Set the value for this element
     *
     * @param value the value for this element
     */
    public void setVal(Object ival) {
        val = ival;
    }

    /**
     * Get the value for this element. If the underlying object
     * is actually an array (ie. allowMultiples = true), then
     * you should really be calling getVals() to get the whole
     * object array; if you call this method, you will just get 
     * the first element of the array.
     *
     * @return the value for this form element (may be null)
     */
    public Object getVal() {
//        return val;
        if (val!=null && val instanceof Object[]) {
            return ((Object[]) val)[0];
        }
        return val;
    }
    
    /**
     * Get all the values for this element. There will always be an array returned.
     * if the value for this FormElement is null, an empty array is returned. This
     * method only really makes sense if allowMultiples = true
     *
     * @return the array of values for this form element
     */
    public Object[] getVals() {
        Object valCopy = val;
        if (valCopy==null) {
            if (type!=null) {
                return type.getTypeArray(0);
            }
            return new Object [] {};
        }
        if (valCopy instanceof ArrayList) {
            valCopy = ((ArrayList)valCopy).toArray();
        } else {
            valCopy =  new Object [] {valCopy};
        }
        Object [] vals = (Object [])valCopy;
        Object [] typedVals = null;
        if (type!=null) {
            typedVals = type.getTypeArray(vals.length);
        }
        if (typedVals==null) {
            typedVals = new Object [vals.length];
        }
        if (logger.isDebugEnabled()) {
            for(int i = 0; i < vals.length; i++) {
                logger.debug("Val: " + vals[i]);
                if (vals[i]!=null) {
                    logger.debug("ValType: " + vals[i].getClass());
                }
            }
        }
        System.arraycopy(vals, 0, typedVals, 0, vals.length);
        return typedVals;
    }

    /**
     * Get the value for this element as a String
     */
    public String getStringVal() {
        return (String) val;
    }

    /**
     * Get the value for this element as a String, defaulting
     * accordingly if the value is null
     */
    public String getStringVal(String dflt) {
        return (val!=null ? (String) val : dflt);
    }

    /**
     * Get the value for this element as a Boolean
     */
    public Boolean getBooleanVal() {
        return (Boolean) val;
    }

    /**
     * Get a Boolean value from the map, defaulting accordingly if
     * the value is null
     */
    public Boolean getBooleanVal(Boolean dflt) {
        return (val!=null ? (Boolean) val : dflt);
    }

    /**
     * Get the value for this element as a Integer
     */
    public Integer getIntegerVal() {
        return (Integer) val;
    }

    /**
     * Get a Integer value from the map, defaulting accordingly if
     * the value is null
     */
    public Integer getIntegerVal(Integer dflt) {
        return (val!=null ? (Integer) val : dflt);
    }

    /**
     * Get the value for this element as a Date
     */
    public Date getDateVal() {
        return (Date) val;
    }

    /**
     * Get a Date value from the map, defaulting accordingly if
     * the value is null
     */
    public Date getDateVal(Date dflt) {
        return (val!=null ? (Date) val : dflt);
    }

    /**
     * Get the value for this element as a Long
     */
    public Long getLongVal() {
        return (Long) val;
    }

    /**
     * Get a Long value from the map, defaulting accordingly if
     * the value is null
     */
    public Long getLongVal(Long dflt) {
        return (val!=null ? (Long) val : dflt);
    }

    /**
     * Get the value for this element as a Short
     */
    public Short getShortVal() {
        return (Short) val;
    }

    /**
     * Get a Short value from the map, defaulting accordingly if
     * the value is null
     */
    public Short getShortVal(Short dflt) {
        return (val!=null ? (Short) val : dflt);
    }

    /**
     * Get the value for this element as a Double
     */
    public Double getDoubleVal() {
        return (Double) val;
    }

    /**
     * Get a Double value from the map, defaulting accordingly if
     * the value is null
     */
    public Double getDoubleVal(Double dflt) {
        return (val!=null ? (Double) val : dflt);
    }

    /**
     * Get the value for this element as a Float
     */
    public Float getFloatVal() {
        return (Float) val;
    }

    /**
     * Get a Float value from the map, defaulting accordingly if
     * the value is null
     */
    public Float getFloatVal(Float dflt) {
        return (val!=null ? (Float) val : dflt);
    }

    /**
     * Set the parse exception associated with this element
     *
     * @return any parse exceptions associated with the element
     */
    public void setParseException(ParseException ipe) {
        pe = ipe;
    }

    /**
     * Get any parse exceptions associated with the element (ie. that
     * might have occurred when the element was mapped)
     *
     * @return any parse exceptions associated with the element
     */
    public ParseException getParseException() {
        return pe;    
    }
    
    /**
     * Get a string representation of this element
     *
     * @return the String describing this element
     */
    public String toString() {
        return "Key:"+key+" Name:"+name+" Val:"+val+" Type:"+(type!=null ? type.getFormClass().getName() : "null")+" Orig:"+origVal+" Dflt:"+defaultVal+" Mult:"+allowMultiples+" Validator:"+(validator!=null ? "@"+Integer.toHexString(validator.hashCode()) : "null");
    }

}
