/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Chris Webb, Diez Roggisch, Iman L. Crawford, Christian Cryder, Jacob Kjome
 *
 * $Id: FormType.java,v 1.26 2003/05/15 18:53:37 christianc Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.math.BigDecimal;
import java.text.*;
import java.sql.Timestamp;
import java.util.*;

import org.enhydra.barracuda.plankton.*;

/**
 * This class defines all valid FormTypes. Currently we support:
 *
 * <ul>
 *        <li>String</li>
 *        <li>Boolean</li>
 *        <li>Integer</li>
 *        <li>Long</li>
 *        <li>Short</li>
 *        <li>Double</li>
 *        <li>Float</li>
 *        <li>BigDecimal</li>
 *        <li>Date</li>
 *        <li>Timestamp</li>
 * </ul>
 * @author  Chris Webb <chris.webb@voxsurf.com>
 * @author  Diez Roggisch <diez.roggisch@artnology.com>
 * @author  Iman L. Crawford <icrawford@greatnation.com>
 * @author  Christian Cryder <christianc@granitepeaks.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public abstract class FormType {

    // String type definition.
    public static FormType STRING = new FormType() {
        public Class getFormClass() {
            return String.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            return origVal;
        }

        public Object [] getTypeArray(int size) {
            return new String [size];
        }
    };

    // Boolean type definition.
    public static FormType BOOLEAN = new FormType() {
        public Class getFormClass() {
            return Boolean.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            String tval = origVal.trim().toLowerCase();
            //csc_061902.1            if (tval.equals("on") || tval.equals("yes") || tval.equals("true")) {
            if (tval.equals("on") || tval.equals("yes") || tval.equals("true") || tval.equals("y")) {   //csc_061902.1
                origVal = "true";
                //csc_061902.1            } else if (tval.equals("off") || tval.equals("no") || tval.equals("false")) {
            } else if (tval.equals("off") || tval.equals("no") || tval.equals("false") || tval.equals("n")) {   //csc_061902.1
                origVal = "false";
            } else {
                throw new ParseException(origVal);
            }
            return new Boolean(origVal);
        }

        public Object [] getTypeArray(int size) {
            return new Boolean [size];
        }
    };

    // Integer type definition.
    public static FormType INTEGER = new FormType() {
        public Class getFormClass() {
            return Integer.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            try {
                return new Integer(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    int d1val = d1.intValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Integer(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
        }
        public Object [] getTypeArray(int size) {
            return new Integer [size];
        }
    };

    // Long type definition.
    public static FormType LONG = new FormType() {
        public Class getFormClass() {
            return Long.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            // ilc_022702.1_start
            // check for 0's after decimal place.
            // val = new Long(origVal);
            try {
                return new Long(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    long d1val = d1.longValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Long(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
            // ilc_022702.1_end
        }

        public Object [] getTypeArray(int size) {
            return new Long [size];
        }
    };

    // Short type definition.
    public static FormType SHORT = new FormType() {
        public Class getFormClass() {
            return Short.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            // ilc_022702.2_start
            // check for 0's after decimal place.
            // val = new Short(origVal);
            try {
                return new Short(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    short d1val = d1.shortValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Short(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
            // ilc_022702.2_end
        }
        public Object [] getTypeArray(int size) {
            return new Short [size];
        }
    };

    // Double type definition.
    public static FormType DOUBLE = new FormType() {
        public Class getFormClass() {
            return Double.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;
            
            try {
                return new Double(origVal);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }
        }

        public Object [] getTypeArray(int size) {
            return new Double [size];
        }
    };

    // Float type definition.
    public static FormType FLOAT = new FormType() {
        public Class getFormClass() {
            return Float.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;
            
            try {
                return new Float(origVal);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }
        }

        public Object [] getTypeArray(int size) {
            return new Float [size];
        }
    };

    // BigDecimal type definition.
    public static FormType BIG_DECIMAL = new FormType() {
        public Class getFormClass() {
            return BigDecimal.class;
        }
            
        public Object parse(String origVal, Locale locale) throws ParseException {
            //csc_060702.1_start - I changed this to use the String constructor since
            //the javadocs say that using the Double constructor is unpredicatable. Note
            //that I also am stripping out the dollar sign if its there (probably still 
            //need to handle other currency symbols, based on locales)
            /*
              Object d = DOUBLE.parse(origVal, locale);
              if (d!=null) {
              return new BigDecimal(((Double)d).doubleValue());
              }
              return null;
            */          
            //csc_040203.2 - wrapped the logic in a block to catch NumberFormatExceptions...not sure why this hadn't been done originally!
            try {
                String s = origVal.trim();
                s = StringUtil.replace(s, "$","");  //strip off $ sign
                s = StringUtil.replace(s, "","");  //strip off  sign
                s = StringUtil.replace(s, ",","");  //strip off commas
                if (s.startsWith("(") && s.endsWith(")")) { //if its a debit (ie. in parenthesis), strip off parenthesis and add a - sign
                    s = "-"+s.substring(1,s.length()-1);
                }
                return new BigDecimal(s);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }


            //csc_060702.1_end 
        }

        public Object [] getTypeArray(int size) {
            return new BigDecimal [size];
        }
    };

    // Date type definition.
    public static FormType DATE = new FormType() {
        public Class getFormClass() {
            return java.util.Date.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null) return null;

            if (locale==null) locale = Locale.getDefault();
            
            // try all the various forms of date format to cover more bases
            try {
                //dbr_011602.1  DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
                DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, locale); //dbr_011602.1
                df.setLenient(false);
                return df.parse(origVal);
            } catch (java.text.ParseException e1) {
                try {
                    //dbr_011602.1      DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);
                    DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM, locale); //dbr_011602.1
                    df.setLenient(false);
                    return df.parse(origVal);
                } catch (java.text.ParseException e2) {
                    try {
                        //dbr_011602.1          DateFormat df = DateFormat.getDateInstance(DateFormat.LONG);
                        DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, locale); //dbr_011602.1
                        df.setLenient(false);
                        return df.parse(origVal);
                    } catch (java.text.ParseException e3) {
                        try {
                            //dbr_011602.1              DateFormat df = DateFormat.getDateInstance(DateFormat.FULL);
                            DateFormat df = DateFormat.getDateInstance(DateFormat.FULL, locale); //dbr_011602.1
                            df.setLenient(false);
                            return df.parse(origVal);
                        } catch (java.text.ParseException e4) {
                            throw new ParseException(e4, "Locale is " + locale.getCountry());
                        }
                    }
                }
                //rtl20010822 - end new
            }
        }

        public Object [] getTypeArray(int size) {
            return new Date [size];
        }
    };

    //csc_051503_1 - added
    // Timestamp type definition.
    public static FormType TIMESTAMP = new FormType() {
        public Class getFormClass() {
            return java.sql.Timestamp.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            Timestamp ts = null;
            Long l = null;

            //first of all, try and see if we can treat it as a Long value
            try {
                l = (Long) LONG.parse(origVal, locale);
            } catch (ParseException e) {}

            //if that didn't work, try to convert it as a date
            if (l==null) {
                Date d = (Date) DATE.parse(origVal, locale);
                l = new Long(d.getTime());
            }

            //finally, try creating the timestamp from the long value
            return new Timestamp(l.longValue());
        }

        public Object [] getTypeArray(int size) {
            return new Timestamp[size];
        }
    };

    /**
     * Protected constructor to prevent external instantiation. Cannot be
     * private because we would be unable to call the constructor from a
     * sub-class.
     */
    protected FormType() { }

    /**
     * Returns the class associated with this particular form type.
     */
    public abstract Class getFormClass();

    /**
     * Parses an object based on the specific form type.
     */
    public Object parse(String origVal) throws ParseException {
        return parse(origVal, null);
    }

    /**
     * Parses an object based on the specific form type.
     */
    public abstract Object parse(String origVal, Locale loc) throws ParseException;


    /** create an array of the FormType's type - if heterogenous types
     * are returned, an array of Object will be returned.
     */
    public abstract Object [] getTypeArray(int size);

    /**
     * Returns a string representation of this particular formt type.
     */
    public String toString() {
        return this.getFormClass().getName();
    }

}
