/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Robert Leftwich, Christian Cryder, Diez B. Roggisch, Jacob Kjome
 *
 * $Id: DateValidator.java,v 1.9 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.*;
import org.apache.log4j.*;

/**
 * This validator ensures that a value is a valid date
 *
 * @author  Robert Leftwich <digital@ix.net.au>
 * @author  Christian Cryder <christianc@barracudamvc.org>
 * @author  Jacob Kjome <hoju@visi.com>
 * @author  Diez B. Roggisch <diez.roggisch@artnology.com>
 * @version %I%, %G%
 * @since   1.0
 */
public class DateValidator extends DefaultFormValidator {

    /**
     * Public constructor.
     *
     */
    public DateValidator() {
        super();
    }

    /**
     * Public constructor.
     *
     * @param errmsg the message associated with this error
     */
    public DateValidator(String ierrmsg) {
        super(ierrmsg);
    }

    /**
     * Validate a FormElement to make sure the value is a valid date
     * Validation is not supported for
     *    FormType.BOOLEAN
     *    FormType,INTEGER
     *    FormType.LONG
     *    FormType.SHORT
     *    FormType.DOUBLE
     *    FormType.FLOAT
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *      to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *      and attempt to validate all elements so that we can process
     *      all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        // ilc_022102.1_start
        //    val is always going to be OrigVal, make no judgment on null values
        if (this.isNull(val, element))
          return;
        // ilc_022102.1_end

        // ilc_022102.2_start
        // comment out we already using origVal.
        /*
        // dbr_122801.1_start If the original Value wasn't null, there might have been a parsing excpetion
        Object oVal = element.getOrigVal();
        if (oVal!=null
            && oVal instanceof String
            && !((String)oVal).trim().equals("")
            && element.getParseException()!=null) {
                throw this.generateException(element,
                            deferExceptions,
                            oVal +  " is not a valid date because " + element.getParseException());
        }
        // dbr_122801.1_end

        // cannot validate a null or empty string value, use NotNullvalidator if you want to disallow nulls
        if (val==null || val.toString().length()==0) return;
        */
        // ilc_022102.2_end

        //if element is null, or not form type=STRING or DATE, err because this should
        //not be considered a valid option (note that these exceptions are always
        //immediate since they typically represent a programming error)
        if (element==null) throw new ValidationException(val, "Object val:"+val+" is associated with a null FormElement");

        FormType formType = element.getType();
        if (!(formType.equals(FormType.DATE) || formType.equals(FormType.STRING))) {
            throw new ValidationException(val, "Unsupported validation: "+
                        val+" is of FormType " +
                        formType.toString() +
                        " and cannot be validated by this validator");
        }


        // ilc_022102.3_start
        // testing against origVal it should always be a string, but test cases
        // want to test against Date
        if (val instanceof Date){
          return;
        }
        else if (formType.equals(FormType.DATE) && element.getParseException()!=null) {
            throw this.generateException(element, deferExceptions, val.toString()
                                                                   +  " is not a valid date, because "
                                                                   + element.getParseException());
        }
        else if (formType.equals(FormType.STRING)) {
          try {
            Date dateVal = (Date)FormType.DATE.parse(val.toString());
            if (localLogger.isInfoEnabled())
              localLogger.info("Date val = " + dateVal.toString());
          } catch (ParseException ex) {
            throw this.generateException(element, deferExceptions, val.toString()
                                                                   +  " is not a valid date, because "
                                                                   + ex);
          }
        }
        // ilc_022102.3_end

        // ilc_022102.4_start
        // commented out the followin and replace with the above
        /*
        if (formType.equals(FormType.DATE)) {
            return;
        } else if (formType.equals(FormType.STRING)) {
            // attempt to convert it to a date type
            try {
                Date dateVal = (Date)FormType.DATE.parse(val.toString());
                if (localLogger.isInfoEnabled()) localLogger.info("Date val = " + dateVal.toString());
            } catch (ParseException ex) {
                throw this.generateException(element,
                            deferExceptions,
                            val.toString() +  " is not a valid date, because " + ex);
            }
        }
        */
        // ilc_022102.4_end
    }
}
