/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DOMLoader.java,v 1.6 2003/06/02 19:09:28 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.IOException;
import java.util.Locale;
import org.w3c.dom.Document;
import org.enhydra.barracuda.core.view.ViewCapabilities;


/**
 * This interface defines the methods needed to implement a DOMLoader.
 */
public interface DOMLoader {

    /**
     * Get a DOM associated with a given class of object
     *
     * @param clazz the class to be loaded as a DOM object
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz) throws IOException;

    /**
     * Get a DOM associated with a given class of object, based on
     * the specified locale
     *
     * @param clazz the class to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz, Locale locale) throws IOException;

    /**
     * Get a DOM associated with a given class of object, based on the
     * specified ViewCapabilities
     *
     * @param clazz the class to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz, ViewCapabilities vc) throws IOException;

    /**
     * Get a DOM associated with a given class name of object
     *
     * @param className the class name to be loaded as a DOM object
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className) throws IOException;

    /**
     * Get a DOM associated with a given class name of object, based on
     * the specified locale
     *
     * @param className the class name to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className, Locale locale) throws IOException;

    /**
     * Get a DOM associated with a given class of object, based on the
     * specified ViewCapabilities
     *
     * @param className the class name to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className, ViewCapabilities vc) throws IOException;

    /**
     * Get a DOM associated with a given document file.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath) throws IOException;

    /**
     * Get a DOM associated with a given document file, based on
     * the specified locale.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath, Locale locale) throws IOException;

    /**
     * Get a DOM associated with a given document file, based on the
     * specified ViewCapabilities.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath, ViewCapabilities vc) throws IOException;

    /**
     * Specify the default DOM factory. This can also be set via
     * the assembly descriptor with &lt;dom-loader&gt;
     *
     * @param df the DOMFactory
     * @see org.enhydra.barracuda.core.event.DefaultApplicationAssembler
     */
    public void setDefaultDOMFactory(DOMFactory df);

    /**
     * Register a DOMFactory for a given class of DOM object.  This can also be set via
     * the assembly descriptor with &lt;dom-loader-register&gt;
     *
     * @param df the DOMFactory
     * @param clazz the class of DOM object to be handled by this factory
     * @see org.enhydra.barracuda.core.event.DefaultApplicationAssembler
     */
    public void registerDOMFactory(DOMFactory df, Class clazz);

    /**
     * Deregister a DOMFactory for a given class of DOM object.
     *
     * @param clazz the class of DOM object to be removed from handling by its registered factory
     */
    public void deregisterDOMFactory(Class clazz);

}
