/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DefaultDOMLoader.java,v 1.16 2003/06/03 15:39:30 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.IOException;
import java.util.Locale;
import java.util.HashMap;
import java.util.Map;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.enhydra.barracuda.core.view.ViewCapabilities;


/**
 * Default implementation of a DOMLoader.
 */
public class DefaultDOMLoader implements DOMLoader {

    protected static final Logger logger = Logger.getLogger(DefaultDOMLoader.class.getName());

    /**
     * @see #getGlobalInstance()
     */
    protected static DefaultDOMLoader globalLoader = null;

    /**
     * arbitrary object used to synchronize upon
     */
    protected static final Object sync = new Object();

    /**
     * used for flagging whether the globalLoader has been initialized and
     * avoiding unnecessary instantiation and synchronization.
     */
    protected static boolean initialized = false;

    /**
     * @see #setDefaultDOMFactory(DOMFactory)
     */
    protected DOMFactory defaultDOMFactory = null;
    protected Map factories = null;
    protected Map classmap = null;

    /**
     * Make sure no one can directly instantiate this class using the
     * default constructor.  To get an instance of this class, one must use
     * {@link getGlobalInstance()}
     */
    private DefaultDOMLoader() {
        defaultDOMFactory = new XMLCDeferredParsingDOMFactory();
        factories = new HashMap();
        classmap = new HashMap();
    }

    /**
     * Get a DOM associated with a given class of object.
     *
     * @param clazz the class to be loaded as a DOM object
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz) throws IOException {
        return _getDOM(clazz, null, null);
    }

    /**
     * Get a DOM associated with a given class of object, based on
     * the specified locale
     *
     * @param clazz the class to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz, Locale locale) throws IOException {
        return _getDOM(clazz, locale, null);
    }

    /**
     * Get a DOM associated with a given class of object, based on the
     * specified ViewCapabilities
     *
     * @param clazz the class to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given class of object
     * @throws IOException
     */
    public Document getDOM(Class clazz, ViewCapabilities vc) throws IOException {
        return _getDOM(clazz, null, vc);
    }

    /**
     * Get a DOM associated with a given class name of object.
     *
     * @param className the class name to be loaded as a DOM object
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className) throws IOException {
        return _getDOM(className, null, null);
    }

    /**
     * Get a DOM associated with a given class name of object, based on
     * the specified locale
     *
     * @param className the class name to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className, Locale locale) throws IOException {
        return _getDOM(className, locale, null);
    }

    /**
     * Get a DOM associated with a given class of object, based on the
     * specified ViewCapabilities
     *
     * @param className the class name to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given class name of object
     * @throws IOException
     */
    public Document getDOM(String className, ViewCapabilities vc) throws IOException {
        return _getDOM(className, null, vc);
    }

    /**
     * Get a DOM associated with a given document file.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath) throws IOException {
        return _getDOMFromFile(docPath, null, null);
    }

    /**
     * Get a DOM associated with a given document file, based on
     * the specified locale.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @param locale the target locale (may be null)
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath, Locale locale) throws IOException {
        return _getDOMFromFile(docPath, locale, null);
    }

    /**
     * Get a DOM associated with a given document file, based on the
     * specified ViewCapabilities.
     * <p>Note that the behavior of this method depends
     * upon the dom factory implementation being used. Some dom
     * factories may not support this method and just throw an IOException.
     * Those that support this method may require the docPath as an OS-specifc
     * hardcoded path, a path relative to a known hardcoded path, a path to the
     * file within the classloader, or anything else you can imagine. See the
     * javadoc of the dom factory implementation you are using for details.</p>
     *
     * @param docPath the document file to be loaded as a DOM object
     * @param vc the clients ViewCapabilities (may be null)
     * @return a DOM for the given document file
     * @throws IOException
     */
    public Document getDOMFromFile(String docPath, ViewCapabilities vc) throws IOException {
        return _getDOMFromFile(docPath, null, vc);
    }


    private Document _getDOM(Class clazz, Locale locale, ViewCapabilities vc) throws IOException {
        //eliminate the obvious
        if (clazz==null) throw new IOException("Invalid class:"+clazz);
        return _getDOM(clazz.getName(), locale, vc);
    }

    private Document _getDOM(String className, Locale locale, ViewCapabilities vc) throws IOException {
        //eliminate the obvious
        if (className==null) throw new IOException("Invalid class name:"+className);

        locale = doGetLocaleCheck(locale, vc);

        //first we need to figure out the real class name (taking into account
        //the locale). To do this, we must understand the naming convention. As
        //with resource bundles, we'll look for class name + locale. For example,
        //if we request HelloWorldHTML.class, but the locale is Spanish (es),
        //then we should get back an instance of the DOM for HelloWorldHTML_es.class,
        //unless of course that class does not exist, in which case it should return
        //the DOM for HelloWorld.class. In this way, the DOM classes themselves act
        //like resource bundles.

        String baseName = className;
        Class targetClass = null;
        String targetName = null;
        String language = locale.getLanguage();
        String country = locale.getCountry();
        String variant = locale.getVariant();
        targetName = baseName+"_"+language+"_"+country+"_"+variant;
        targetClass = lookupClass(targetName);
        if (targetClass==null) {
            targetName = baseName+"_"+language+"_"+country;
            targetClass = lookupClass(targetName);
        }
        if (targetClass==null) {
            targetName = baseName+"_"+language;
            targetClass = lookupClass(targetName);
        }
        if (targetClass==null) {
            targetName = baseName;
            targetClass = lookupClass(targetName);
        }
        if (logger.isDebugEnabled()) logger.debug("Target class: "+targetClass);
        if (targetClass==null) return null;

        //at this point we should have a valid class; now we
        //need to create a DOM for it
        synchronized (factories) {
            if (logger.isDebugEnabled()) logger.debug("Loading DOM");
            DOMFactory df = (DOMFactory) factories.get(targetClass);
            if (df!=null) {
                return df.getInstance(targetClass);
            } else {
                return defaultDOMFactory.getInstance(targetClass);
            }
        }
    }

    private Document _getDOMFromFile(String docPath, Locale locale, ViewCapabilities vc) throws IOException {
        //eliminate the obvious
        if (docPath==null) throw new IOException("Invalid document path:"+docPath);
        return defaultDOMFactory.getInstance(docPath);
    }


    private static Locale doGetLocaleCheck(Locale locale, ViewCapabilities vc) throws IOException {
        //if the locale is null, look to the ViewCapabilities
        if (locale==null && vc!=null) {
            locale = vc.getClientLocale();
            if (logger.isDebugEnabled()) logger.debug("Retrieving locale from ViewCapabilities: "+locale);
        }

        //if the locale is still null, just use the default
        if (locale==null) {
            locale = Locale.getDefault();
            if (logger.isDebugEnabled()) logger.debug("Using default locale: "+locale);
        }
        return locale;
    }

    /**
     * Find a stored reference to a class. If not found, instantiate it
     * and store it for later reference and faster retrieval.
     *
     * @param className the fully qualified name of a class
     * @return the instantiated class
     */
    protected Class lookupClass(String className) {
        synchronized (classmap) {
            //first see if we have a reference to that class in our classmap
            Class clazz = (Class) classmap.get(className);

            //if not, try to instantiate it
            if (clazz==null) {
                try {
                    clazz = Class.forName(className, true, Thread.currentThread().getContextClassLoader());
                    classmap.put(className, clazz);
                } catch (Exception e) {}
            }
            return clazz;
        }
    }

    /**
     * Specify the default DOM factory. This can also be set via
     * the assembly descriptor with &lt;dom-loader&gt;
     *
     * @param df the DOMFactory
     * @see org.enhydra.barracuda.core.event.DefaultApplicationAssembler
     */
    public void setDefaultDOMFactory(DOMFactory df) {
        if (logger.isDebugEnabled()) logger.debug("Setting default DOM factory:"+df);
        synchronized (factories) {
            defaultDOMFactory = df;
        }
    }

    /**
     * Register a DOMFactory for a given class of DOM object.  This can also be set via
     * the assembly descriptor with &lt;dom-loader-register&gt;
     *
     * @param df the DOMFactory
     * @param clazz the class of DOM object to be handled by this factory
     * @see org.enhydra.barracuda.core.event.DefaultApplicationAssembler
     */
    public void registerDOMFactory(DOMFactory df, Class clazz) {
        if (logger.isDebugEnabled()) logger.debug("Registering DOM factory:"+df+" for class:"+clazz);
        synchronized (factories) {
            factories.put(clazz, df);
        }
    }

    /**
     * Deregister a DOMFactory for a given class of DOM object.
     *
     * @param clazz the class of DOM object to be removed from handling by its registered factory
     */
    public void deregisterDOMFactory(Class clazz) {
        if (logger.isDebugEnabled()) logger.debug("Deregistering DOM factory for class:"+clazz);
        synchronized (factories) {
            factories.remove(clazz);
        }
    }

    /**
     * Get the global instance of the DOMLoader
     *
     * @return the global instance of the DOMLoader
     */
    public static DOMLoader getGlobalInstance() {
        if (initialized == false) {
            synchronized (sync) {
                logger.info("initializing global instance of DefaultDOMLoader");
                globalLoader = new DefaultDOMLoader();
                initialized = true;
            }
        }
        return globalLoader;
    }

/*
    public static void main(String args[]) {
        DOMLoader loader = DefaultDOMLoader.getGlobalInstance();
        Class targetCl = org.enhydra.barracuda.tutorials.xmlc.HelloWorld4HTML.class;
        Document dom = null;

        try {
            //load the default
            dom = loader.getDOM(targetCl);
            System.out.println ("DOM:"+dom);

            //load Spanish
            dom = loader.getDOM(targetCl, new Locale("es",""));
            System.out.println ("DOM:"+dom);

            //load Spanish (Chile)
            dom = loader.getDOM(targetCl, new Locale("es","CL"));
            System.out.println ("DOM:"+dom);

            //load French
            dom = loader.getDOM(targetCl, new Locale("fr",""));
            System.out.println ("DOM:"+dom);

            //load Arabic (should return default)
            dom = loader.getDOM(targetCl, new Locale("ar",""));
            System.out.println ("DOM:"+dom);

        } catch (Exception e) {
            System.out.println ("Err:"+e);
        }
    }
*/

}
