/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DefaultDOMWriter.java,v 1.17 2003/06/23 15:39:09 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.w3c.tidy.Tidy;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.xml.io.*;


/**
 * This class provides the default implementation for writing a DOM.
 * XMLC is used for formatting the dom.  Besides output options features
 * supplied by XMLC, supported features include pretty printing and preventing
 * page caching (which prevents any redisplay of the page even via the back
 * button) or setting a max-age (which generally allows for redisplay of a
 * page via the back button, but not when revisiting the URL).  So, the prevent
 * caching and max age features are mutually exclusive. Default behavior is for
 * default output options (obtained automatically from the current document),
 * no pretty printing, and setting a max-age header of 0 (zero).  Defaults may
 * be modified via class static variables or overridden explicitly via
 * constructors and/or mutators.
 */
public class DefaultDOMWriter implements DOMWriter {

    protected static final Logger logger = Logger.getLogger(DefaultDOMWriter.class.getName());

    /**
     * default value for pretty printing, false unless modified at runtime
     */
    public static boolean defaultPrintPretty = false;

    /**
     * default value for preventing caching, false unless modified at runtime
     */
    public static boolean defaultPreventCaching = false;

    /**
     * default value for max age of rendered page, 0 (zero) unless modified at runtime
     */
    public static int defaultMaxAge = 0;

    protected DOMFormatter dfm = null;
    protected OutputOptions oo = null;
    protected boolean printPretty = false;
    protected boolean preventCaching = false;
    protected boolean leaveWriterOpen = false;
    protected int maxAge = 0;   //csc_061202.1 - added

    /**
     * Default constructor
     */
    public DefaultDOMWriter() {
        this(null, defaultPrintPretty, defaultPreventCaching, defaultMaxAge);
    }

    /**
     * Public constructor. Allows you to specify OutputOptions.
     *
     * @param oo OutputOptions to specify how the DOM should be formatted
     */
    public DefaultDOMWriter(OutputOptions oo) {
        this(oo, defaultPrintPretty, defaultPreventCaching, defaultMaxAge);
    }

    /**
     * Public constructor. Allows you to specify pretty printing.
     *
     * @param printPretty true if pretty print
     */
    public DefaultDOMWriter(boolean printPretty) {
        this(null, printPretty, defaultPreventCaching, defaultMaxAge);
    }

    /**
     * Public constructor. Allows you to specify output options,
     * pretty printing and whether or not the page should be cached or the
     * max-age header for the page (how many seconds until it expires).
     * Note that the latter two options are mutually exclusive.
     *
     * @param ioo OutputOptions to specify how the DOM should be formatted, may be null
     * @param iprintPretty true if pretty print
     * @param ipreventCaching true if we want to prevent client side caching of the page
     * @param imaxAge how many seconds until the page should expire
     */
    public DefaultDOMWriter(OutputOptions ioo, boolean iprintPretty, boolean ipreventCaching, int imaxAge) {
        setOutputOptions(ioo);
        setPrettyPrint(iprintPretty);
        setPreventCaching(ipreventCaching);
        setMaxAge(imaxAge);
    }

    /**
     * Write a DOM to a ServletResponse object. This method will
     * automatically set the content type for you.
     *
     * @param node the DOM node to be written out
     * @param resp the HttpServletResponse object
     */
    public void write(Node node, HttpServletResponse resp) throws IOException {
        //set the response type (TODO:need to support WML as well)
        String responseType = "text/xml";
        if (node instanceof HTMLDocument) responseType = "text/html";
        if (logger.isDebugEnabled()) logger.debug("Setting the response type:"+responseType);
        resp.setContentType(responseType);

        //if we need to prevent caching
        if (preventCaching) {
            //add the appropriate headers to the response
            if (logger.isDebugEnabled()) logger.debug("updating resp hdr to prevent caching");
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());

        //csc_061202.1 - added
        //otherwise explicitly give it a max-age (this will generally allow browsers like
        //IE to page back in history without reloading, but if the user actually revisits the
        //URL, then it will still be reloaded)
        } else {
            resp.setHeader("Cache-Control","max-age="+maxAge);
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());
        }

        //write the dom
        write(node, resp.getWriter());
    }

    /**
     * Write a DOM to an OutputStream.
     *
     * @param node the DOM node to be written out
     * @param out the OutputStream to be written to
     */
    public void write(Node node, OutputStream out) throws IOException {
        write(node, new PrintWriter(out));
    }

    /**
     * Write a DOM to a Writer.
     *
     * @param node the DOM node to be written out
     * @param writer the writer to be written to
     */
    public void write(Node node, Writer writer) throws IOException {
        //build the default DOMFormatter if necessary
        if (dfm==null) dfm = new DOMFormatter();

        OutputOptions localoo = null;
        //set the default output options if necessary
        if (oo != null) localoo = oo;
        if (localoo == null) localoo = getDefaultOutputOptions(node.getOwnerDocument());

        //configure the formatter
        dfm.setOutputOptions(localoo);

        //now print...
        if (!printPretty) {
            //write the doc directly to the writer
            if (logger.isDebugEnabled()) logger.debug("Writing DOM directly to writer");
            dfm.write(node, writer);
            if (!leaveWriterOpen) writer.close();
        } else {
            //write the doc to a byte stream
            byte[] bytes = dfm.toBytes(node);

            //filter the byte stream to tidy
            if (logger.isDebugEnabled()) logger.debug("Formatting for pretty print");
            Tidy tidy = new Tidy();
            tidy.setIndentContent(true);
            tidy.setSpaces(2);
            tidy.setQuiet(true);
            ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            tidy.parse(bais, baos);

            //now write the file
            if (logger.isDebugEnabled()) logger.debug("Writing DOM to writer");
            writer.write(baos.toString());
            if (!leaveWriterOpen) writer.close();
        }
    }

    public void setPrettyPrint(boolean val) {
        this.printPretty = val;
    }

    public void setPreventCaching(boolean val) {
        this.preventCaching = val;
    }

    //csc_061202.1 - added
    public void setMaxAge(int imax) {
        this.maxAge = imax;
    }

    //jrk_20030317.1
    public void setOutputOptions(OutputOptions ioo) {
        this.oo = ioo;
    }

    public void setLeaveWriterOpen(boolean val) {
        leaveWriterOpen = val;
    }

    public static OutputOptions getDefaultOutputOptions(Document doc) {
        return DOMFormatter.getDefaultOutputOptions(doc);
    }

/*
    public static void main(String args[]) {
        //setup
        DOMWriter dw = new DefaultDOMWriter();
        org.enhydra.xml.xmlc.XMLCFactory xmlcFactory = new org.enhydra.xml.xmlc.XMLCStdFactory(dw.getClass().getClassLoader(), new org.enhydra.xml.xmlc.StreamXMLCLogger());
        org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML page1 = null;
        org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML page2 = null;
        StringWriter sw = null;

        //load & print DOM page
System.out.println ("");
System.out.println ("Here 0");
        page1 = (org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML) xmlcFactory.create(org.enhydra.barracuda.tutorials.xmlc.Visibility1HTML.class);
        page1.getElementItem2().setAttribute("visdom","false");
        page1.getElementItem4().setAttribute("visdom","false");
        sw = new StringWriter(5000);
        try {
            dw.write(page1, sw);
        } catch (IOException e){
            System.out.println("Error writing DOM:"+e);
        }
        System.out.println(sw.toString());

        //load & print DOM page
System.out.println ("");
System.out.println ("Here 1");
        page2 = (org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML) xmlcFactory.create(org.enhydra.barracuda.tutorials.xmlc.Visibility2HTML.class);
        page2.getElementItem2().setAttribute("visdom","false");
        page2.getElementItem4().setAttribute("visdom","false");
        sw = new StringWriter(5000);
        try {
            dw.write(page2, sw);
        } catch (IOException e){
            System.out.println("Error writing DOM:"+e);
        }
        System.out.println(sw.toString());
    }

*/

}
