/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SessionServices.java,v 1.15 2002/11/04 08:15:33 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.http;

import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.ViewContext;
import org.enhydra.barracuda.core.event.ControlEventContext;
import org.enhydra.barracuda.plankton.data.ReferenceFactory;


/**
 * This class defines a convenience method to get the session 
 * and set the timeout at the same time. It also provides a 
 * mechanism to easily cache objects in the session by Reference
 * (which allows them to automatically be removed from the session
 * if the system starts running low on memory)
 */
public class SessionServices extends org.enhydra.barracuda.plankton.http.SessionServices {

    /**
     * get the session from the view context (it will automatically extract
     * the event context and cast it to ControlEventContext for you, then 
     * retrieve the request from there and use that to get the session info). 
     * If no session exists it will create it for us. Ensures that the timeout 
     * is set to DEFAULT_TIMEOUT.
     *
     * @param vc the ViewContext
     * @return the users session
     */
    public static HttpSession getSession(ViewContext vc) {
        return getSession(((ControlEventContext) vc.getEventContext()).getRequest());
    }
    
    /**
     * get the session from the view context (allowing you to specify whether or 
     * not to create it)
     *
     * @param vc the ViewContext
     * @param boolean if true, the session will be created if it does not
     *      already exist
     * @return the users session (may be null if the session does not yet
     *      exist and create is false)
     */
    public static HttpSession getSession(ViewContext vc, boolean create) {
        return getSession(((ControlEventContext) vc.getEventContext()).getRequest(), create);
    }
    
    /**
     * get the session from the view context (allowing you to specify 
     * whether or not to create it). If the session exists, it 
     * will set the default timeout for us.
     *
     * @param vc the ViewContext
     * @param create if true, the session will be created if it does not
     *      already exist
     * @param timeout the default timeout value (null indicates do not set)
     * @return the users session (may be null if the session does not yet
     *      exist and create is false)
     */
    public static HttpSession getSession(ViewContext vc, boolean create, Integer timeout) {
        return getSession(((ControlEventContext) vc.getEventContext()).getRequest(), create, timeout);
    }


    /**
     * This method retrieves the session from the Context, and then looks for an 
     * object in the session based on a given key. If the object is not present, 
     * it will be created using the ReferenceFactory and cached in session for 
     * future use.
     *
     * @param context the ControlEventContext
     * @param key the key that identifies this object
     * @param factory the ReferenceFactory used to create the object
     * @return the object from the cache 
     */
    public static Object getObjectFromCache(ControlEventContext context, Object key, ReferenceFactory factory) {
        HttpSession session = SessionServices.getSession(context.getRequest());
        return getObjectFromCache(session, key, factory);
    }
    
}
