/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Locales.java,v 1.6 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.l10n;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.event.*;


/**
 * Simple locale utilities. This class makes it easy to determine the target
 * locale from an event context or a servlet request. You can also set the locale
 * and ask the class to save the information for you (in a cookie and/or the
 * session) so that the locale info will persist across muliple requests
 */
public class Locales extends org.enhydra.barracuda.plankton.l10n.Locales {

    protected static final Logger logger = Logger.getLogger(Locales.class.getName());

    /**
     * Get the client locale from an EventContext using the default
     * param keys and persist option. This will probably be the method
     * you use most frequently.
     *
     * @param ec the EventContext from which we'd like to determine Locale
     * @return the target client Locale
     */
    public static Locale getClientLocale(EventContext ec) {
        return getClientLocale(ec, LANGAUGE_KEY, COUNTRY_KEY, VARIANT_KEY, PERSIST_DEFAULT);
    }

    /**
     * Get the client locale from an EventContext
     *
     * @param ec the EventContext from which we'd like to determine Locale
     * @param languageKey the key to be used to look in the request for a
     *        language paramter
     * @param countryKey the key to be used to look in the request for a
     *        country paramter
     * @param variantKey the key to be used to look in the request for a
     *        variant paramter
     * @param persistOption how we'd like to persist the Locale (by default, it will be stored
     *        in the SESSION)
     * @return the target client Locale
     */
    public static Locale getClientLocale(EventContext ec, String languageKey, String countryKey, String variantKey, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        return getClientLocale(req, resp, languageKey, countryKey, variantKey, persistOption);
    }

    /**
     * Save the client locale using an EventContext using the default
     * persist option
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param loc the target client locale we'd like to set
     */
    public static void saveClientLocale(EventContext ec, Locale loc) {
        saveClientLocale(ec, loc, PERSIST_DEFAULT);
    }

    /**
     * Save the client locale using an EventContext
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param loc the target client locale we'd like to set
     * @param persistOption the specific persistOption to be used
     */
    public static void saveClientLocale(EventContext ec, Locale loc, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        saveClientLocale(req, resp, loc, persistOption);
    }

    /**
     * Release the client locale using an EventContext (this effectively
     * removes it from whereever it might have been persisted). This means that
     * on the next request the locale will be determined from scratch again.
     *
     * @param ec the EventContext in which we'd like to set Locale (this tells us
     *        where the locale info needs to be removed from)
     */
    public static void releaseClientLocale(EventContext ec) {
        releaseClientLocale(ec, PERSIST_DEFAULT);
    }

    /**
     * Release the client locale using an EventContext (this effectively
     * removes it from whereever it might have been persisted). This means that
     * on the next request the locale will be determined from scratch again.
     *
     * @param ec the EventContext in which we'd like to set Locale
     * @param persistOption the specific persistOption to be used (this tells us
     *        where the locale info needs to be removed from)
     */
    public static void releaseClientLocale(EventContext ec, int persistOption) {
        HttpServletRequest req = (HttpServletRequest) ec.getState(ControlEventContext.HTTP_SERVLET_REQUEST);
        HttpServletResponse resp = (HttpServletResponse) ec.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);
        releaseClientLocale(req, resp);
    }
}
