/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SimpleServiceFinder.java,v 1.13 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.srv;

import java.awt.Container;
import java.util.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.plankton.srv.SimpleServiceProvider;

/**
 * <p>Similar to org.enhydra.barracuda.plankton.srv.SimpleServiceProvider,
 * except that it will also search EventGateway heirarchies.
 */
public class SimpleServiceFinder extends org.enhydra.barracuda.plankton.srv.SimpleServiceFinder {

    protected static final Logger logger = Logger.getLogger(SimpleServiceFinder.class.getName());

    //public constants

    /**
     * Find an instance of a Class in an EventGateway heirarchy. Search
     * direction defaults to UPSTREAM
     *
     * @param    c the class we're looking for
     * @param    gateway the entry point to the EventGateway heirarchy
     * @return    the first instance of the specified class
     */
    public static Object findInstance (Class c, EventGateway gateway) {
        return findInstance_upstream(c, gateway);    
    }
    
    /**
     * Find an instance of a Class in an EventGateway heirarchy
     *
     * @param    c the class we're looking for
     * @param    gateway the entry point to the EventGateway heirarchy
     * @param    searchDirection the search direction
     * @return    the first instance of the specified class
     */
    public static Object findInstance (Class c, EventGateway gateway, int searchDirection) {
        if (searchDirection==DOWNSTREAM) return findInstance_downstream(c, gateway);
        else return findInstance_upstream(c, gateway);    
    }

    /**
     * the private method to actually find the desired object
     */
    private static Object findInstance_upstream (Class c, Object parent) {
        //eliminate the obvious
        if (c==null || parent==null) return null;
        if (logger.isDebugEnabled()) logger.debug("Looking for instance of "+c+" in "+parent);

        //see if the parent object matches
        if (c.isInstance(parent)) return parent;

        //now see if we can get an iterator from this object
        Iterator it = null;
        if (parent instanceof Map) it = ((Map) parent).values().iterator();
        else if (parent instanceof List) it = ((List) parent).iterator();
        else if (parent instanceof SimpleServiceProvider) it = ((SimpleServiceProvider) parent).getSupportedServices().iterator();
        if (it!=null) while (it.hasNext()) {
            Object o = it.next();
            if (logger.isDebugEnabled()) logger.debug("Evaluating iterator item: "+o);
            if (c.isInstance(o)) return o;
        }
        
        //next see if parent is a Container    
        if (parent instanceof Container) {
            Object o[] = ((Container) parent).getComponents();
            if (o!=null) for (int i=0, max=o.length; i<max; i++) {
                if (logger.isDebugEnabled()) logger.debug("Evaluating container item: "+o[i]);
                if (c.isInstance(o[i])) return o[i];
            }
        }
        
        //finally inspect the Parents     
        Object gramps = null;
        if (parent instanceof PData) gramps = ((PData) parent).getParent();
        else if (parent instanceof EventGateway) gramps = ((EventGateway) parent).getParent();
        else if (parent instanceof Container) gramps = ((Container) parent).getParent();
        if (logger.isDebugEnabled()) logger.debug("Evaluating Gramps:"+gramps);
        if (gramps==null) return null;
        else return findInstance_upstream(c, gramps);
    }

    /**
     * the private method to actually find the desired downstream object
     */
    private static Object findInstance_downstream (Class c, Object child) {
        //eliminate the obvious
        if (c==null || child==null) return null;
        if (logger.isDebugEnabled()) logger.debug("Looking for instance of "+c+" in "+child);

        //see if the parent object matches
        if (c.isInstance(child)) return child;

        //search downstream
        //...in SimpleServiceProviders
        if (child instanceof SimpleServiceProvider) {
            Iterator it = ((SimpleServiceProvider) child).getSupportedServices().iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (logger.isDebugEnabled()) logger.debug("Evaluating iterator item: "+o);
                
                //evaluate the object itself
                if (c.isInstance(o)) return o;
                
                //search down the object branch
                Object inst = findInstance_downstream(c, o);
                if (inst!=null) return inst;
            }
        }
        //...in EventGateways
        if (child instanceof EventGateway) {
            Iterator it = ((EventGateway) child).getChildren().iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (logger.isDebugEnabled()) logger.debug("Evaluating iterator item: "+o);
                
                //evaluate the object itself
                if (c.isInstance(o)) return o;
                
                //search down the object branch
                Object inst = findInstance_downstream(c, o);
                if (inst!=null) return inst;
            }
        }
        //...in Lists
        if (child instanceof List) {
            Iterator it = ((List) child).iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (logger.isDebugEnabled()) logger.debug("Evaluating iterator item: "+o);
                
                //evaluate the object itself
                if (c.isInstance(o)) return o;
                
                //search down the object branch
                Object inst = findInstance_downstream(c, o);
                if (inst!=null) return inst;
            }
        }
        //...in Maps
        if (child instanceof Map) {
            Iterator it = ((Map) child).values().iterator();
            while (it.hasNext()) {
                Object o = it.next();
                if (logger.isDebugEnabled()) logger.debug("Evaluating iterator item: "+o);
                
                //evaluate the object itself
                if (c.isInstance(o)) return o;
                
                //search down the object branch
                Object inst = findInstance_downstream(c, o);
                if (inst!=null) return inst;
            }
        }
        //...in Containers
        if (child instanceof Container) {
            Object o[] = ((Container) child).getComponents();
            if (o!=null) for (int i=0, max=o.length; i<max; i++) {
                if (logger.isDebugEnabled()) logger.debug("Evaluating container item: "+o[i]);

                //evaluate the object itself
                if (c.isInstance(o[i])) return o[i];

                //search down the object branch
                Object inst = findInstance_downstream(c, o[i]);
                if (inst!=null) return inst;
            }
        }
        
        //if we didn't find anything return null;
        return null;
    }

}
