/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ObjectRepositoryAssembler.java,v 1.7 2003/06/21 01:04:49 christianc Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.apache.log4j.*;
import org.apache.log4j.xml.*;

import org.enhydra.barracuda.plankton.*;
import org.enhydra.barracuda.plankton.xml.*;


/**
 * This class basically assembles objects into the default ObjectRepository based on
 * an XML descriptor file. For a sample file look at /WEB-INF/object-repository.xml. 
 * This class demonstrates all the basic functionality, so please be sure to look there 
 * first if you have questions. You may want to run TestObjectRepository.xml to see
 * it in action.
 *
 * <p>In a nutshell, the ObjectRepositoryAssembler is a lightweight scripting mechanism 
 * that makes it possible to instantiate objects, set properties, invoke methods, and 
 * then register objects in the global object repository if you so desire (hence the 
 * class name). This class is extremely useful for configuration, so it will be well 
 * worth your time to learn how it works.
 *
 * <p>There are only a few basic tags:
 * <ul>
 *   <li>
 *      <strong>object</strong> - this tag asks the assembler to instantiate an
 *          object, using the name, class, and arg parameters specified. The object
 *          will be instantiated directly after the start tag is processed (ie. before
 *          any method tags, since the object must exist in order for the methods
 *          to be invoked). If the class referenced has no public constructors, a reference
 *          to the class will be saved, allowing you to invoke public static methods.
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - If you specify an object name (ie. '$obj1'),
 *              a reference to the object will be saved for the duration of the
 *              assembly process, allowing you to refer to the object elsewhere
 *              (ie. as a parameter of another method). Note that $this is a reserved
 *              name (the only one actually), which will give you a handle to the 
 *              ObjectRepositoryAssembler object, which also happens to be a servlet, thus
 *              making it possible to grab a reference to the ServletConfig object (if you
 *              are assembling in Standalone Application mode, the ServletConfig object
 *              will obviously not be avialable)
 *        </li><li>
 *           <strong>class</strong> - The name of the object to be instantiated. Note that
 *              if no class name is specified, the assembler looks in the object map
 *              for an existing object that matches of this name. This means that if you
 *              specify the class attribute, you will necessarily be forcing instantiation.
 *              So if you want to invoke methods on an existing object, DON'T provide the 
 *              class name) 
 *        </li><li>
 *           <strong>arg, arg0...arg9</strong> - You can pass args into the object
 *              constructor as well if need be. Frequently, the args might refer to other
 *              named objects.
 *        </li>
 *      </ul>
 *   </li>
 *   <li>
 *      <strong>method</strong> - this tag invokes a method on the object currently
 *          in scope. The method is not actually invoked until the end tag is hit
 *          (allowing us to get any param tags that might accompany the method).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - name of the method to be invoked
 *        </li><li>
 *           <strong>return</strong> - if the method returns an object you can give it 
 *              a return name which will cause the object to be available for later reference 
 *              by the return name
 *        </li><li>
 *           <strong>arg, arg0...arg9</strong> - You can pass args into the object
 *              constructor as well if need. Frequently, the args might refer to other
 *              named objects.
 *        </li>
 *      </ul>
 *
 *      <p>In addition to using arg attributes, you can use the value of the method
 *          tag (ie. if you only have one param), or you can use multiple param
 *          tags.
 *   </li>
 *   <li>
 *      <strong>prop</strong> - this tag allows you to set a property in an object (static
 *          or otherwise). The property is not actually set until the end tag is hit
 *          (allowing us to get any param tags that might accompany the prop).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>name</strong> - name of the property to be set
 *        </li>
 *      </ul>
 *
 *      <p>Unlike the method attribute, prop doesn't take arguments. Instead, you use the
 *          value of the prop to specify what the value should be set to. Right now, Strings,
 *          Integers, Shorts, Longs, Doubles, and Booleans are supported. If you need to set 
 *          something of a different type, email the list (its easy to add support for additional
 *          types).
 *   </li>
 *   <li>
 *      <strong>param</strong> - this tag specifies a parameter for a method. Note that
 *          the param type checking is not very sophisticated; if the param is an instance
 *          of a String (as opposed to a reference to a named object), the assembler
 *          will try it first as an int (if possible), else as a String.
 *
 *      <p>Attributes: n/a
 *   </li>
 *   <li>
 *      <strong>register</strong> - this tag asks the assembler to place the specified
 *          object into the default object repository (usually the global obj repos).
 *
 *      <p>Attributes:<ul>
 *        <li>
 *           <strong>key</strong> - the key value which may be used to retrieve the object
 *              from the default object repository
 *        </li><li>
 *           <strong>val</strong> - a reference to the item to be placed in the default
 *              object repository (ie. $obj1)
 *        </li>
 *      </ul>
 *   </li>
 * </ul>
 */
public class ObjectRepositoryAssembler extends HttpServlet {

    //public constants
    protected static final Logger logger = Logger.getLogger(ObjectRepositoryAssembler.class.getName());

    //private constants
    public static String ASSEMBLY_DESCRIPTOR = "AssemblyDescriptor";
    public static String SAX_PARSER = "SAXParser";
    public static String DEFAULT_DESCRIPTOR = "object-repository.xml";
    public static String DEFAULT_PARSER = "org.apache.xerces.parsers.SAXParser";    //not used any more
    public static String LOG_HEARTBEAT_STR = "LogHeartbeat";
    public static String GLOBAL_CONTINUE_ON_ERR = "ContinueAssemblyOnError";               //csc_041603.2

    //recognized tags
    private static final String OBJECT = "object";
    private static final String METHOD = "method";
    private static final String PROP = "prop";
    private static final String PARAM = "param";
    private static final String RETURN = "return";
    private static final String REGISTER = "register";

    //recognized attributes
    private static final String NAME = "name";
    private static final String CLASS = "class";
    private static final String ARG = "arg";
    private static final String KEY = "key";
    private static final String VAL = "val";
    private static final String CONTINUE_ON_ERR = "continue_on_err";        //csc_041603.2

    //reserved objMap vars
    private static final String THIS = "$this";

    //private vars
    protected boolean logHeartbeat = false;
    public static boolean globalContinueOnErr = false; //csc_041603.2

    public static String TEST_STRING = null;
    public static int TEST_INT = -1;
    public static Integer TEST_INT2 = new Integer(-1);
    public static short TEST_SHORT = -1;
    public static Short TEST_SHORT2 = new Short((short)-1);
    public static long TEST_LONG = -1;
    public static Long TEST_LONG2 = new Long(-1);
    public static double TEST_DOUBLE = -1;
    public static Double TEST_DOUBLE2 = new Double(-1);
    public static float TEST_FLOAT = -1;
    public static Float TEST_FLOAT2 = new Float(-1);
    public static boolean TEST_BOOLEAN = false;
    public static Boolean TEST_BOOLEAN2 = Boolean.FALSE;
    public static String TEST_STRING88 = null;
    public static String TEST_STRING99 = null;

    //--------------- ObjectRepositoryAssembler ------------------
    /**
     * Assemble the system into the default ObjectRepository given the
     * XML assembly decriptor name. The default parser will be
     * used.
     *
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(String iassemblySourceFile) {
        assemble(null, iassemblySourceFile);
    }

    /**
     * Assemble the system into the default ObjectRepository given the
     * XML assembly decriptor name. The default parser will be
     * used.
     *
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(ServletConfig iservletConfig, String iassemblySourceFile) {
        assemble(null, iservletConfig, iassemblySourceFile);
    }

    /**
     * Assemble the system, given a specified ObjectRepository and the
     * XML assembly decriptor name. The default parser will be
     * used.
     *
     * @param or the repository we wish to assemble into (if null
     *      defaults to default ObjectRepository)
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     */
    public void assemble(ObjectRepository ior, ServletConfig iservletConfig, String iassemblySourceFile) {
        assemble(ior, iservletConfig, iassemblySourceFile, null);
    }

    /**
     * Assemble the system, given the root EventGateway, an
     * XML assembly decriptor name, and a specific SAX parser
     * class.
     *
     * @param or the repository we wish to assemble into (if null
     *      defaults to default ObjectRepository)
     * @param servletConfig the ServletConfig object (may be null if you are
     *      calling this from other than a servlet environment)
     * @param assemblySourceFile the XML assembly descriptor (if null defaults
     *      to DEFAULT_DESCRIPTOR)
     * @param parserClass the SAX parser factory class (if null, defaults to DEFAULT_PARSER)
     */
    public void assemble(ObjectRepository ior, ServletConfig iservletConfig, String iassemblySourceFile, String iparserClass) {
        if (ior==null) ior = ObjectRepository.getGlobalRepository();
        if (iassemblySourceFile==null) iassemblySourceFile = DEFAULT_DESCRIPTOR;
        if (iparserClass==null) iparserClass = DEFAULT_PARSER;


        InputStream is = null;               //This is the InputStream that is eventually passed to the AssemblerXmlReader.

        try {
            //Setup the assemblerXmlReader
            AssemblerXMLReader assemblerXmlReader = new AssemblerXMLReader(iparserClass, ior);
            assemblerXmlReader.setup();

            String normalizedFilePath = getNormalizedPath(iassemblySourceFile);
            //Attempt to load from the WEB-INF Dir
            if (iservletConfig!=null) {
                //first assume that the file was given the full path specified relative to the root of the webapp.
                //eg... "/WEB-INF/myresource.xml" or "WEB-INF/myresource.xml"
                if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file via servlet context at: /"+normalizedFilePath);
                is = iservletConfig.getServletContext().getResourceAsStream("/" + normalizedFilePath);
                if (is==null) {
                    //maybe only the filename is provided with no extra path. Alternatively, maybe the path
                    //given was a path relative to /WEB-INF/ rather than the root of the webapp. Try loading from /WEB-INF/.
                    if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file via servlet context at: /WEB-INF/"+normalizedFilePath);
                    is = iservletConfig.getServletContext().getResourceAsStream("/WEB-INF/" + normalizedFilePath);
                    if (is==null) {
                        //maybe only the filename is provided with no extra path.  Alternatively, maybe the path
                        //given was a package path, in which case the path would be relative to /WEB-INF/classes/
                        //rather than the root of the webapp. Try loading from /WEB-INF/classes/.
                        if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file via servlet context at: /WEB-INF/classes/"+normalizedFilePath);
                        is = iservletConfig.getServletContext().getResourceAsStream("/WEB-INF/classes/" + normalizedFilePath);
                    }
                }
            }

            //resource does not exist inside the webapp at the path specified.  Try loading via the classloader.
            if (is==null) {
                if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file inside the local classloader: "+normalizedFilePath);
                is = this.getClass().getClassLoader().getResourceAsStream(normalizedFilePath);
                if (is==null) {
                    //this should only get triggered in the case that this class exists in a parent
                    //classloader and the resource exists in a child classloader.  eg... in the WebappClassloader.
                    if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file inside all available classloaders: "+normalizedFilePath);
                    Thread.currentThread().getContextClassLoader().getResourceAsStream(normalizedFilePath);
                }
            }

            //Not a webapp or path specified is outside of the webapp. Attempt to load using the full file path
            if (is==null) {
                if (logger.isDebugEnabled()) logger.debug("Attempting to load assembly file via file IO at: "+iassemblySourceFile);
                File f = new File(iassemblySourceFile);
                is = new FileInputStream(f);
            }

            if (is!=null) {
                String logString = "Configuring "+ior.getName()+" from File: "+iassemblySourceFile;
                if (iservletConfig != null) iservletConfig.getServletContext().log(logString);
                if (logHeartbeat) System.out.println(logString);
                if (logger.isDebugEnabled()) logger.debug(logString);
            }
            assemblerXmlReader.processXmlFile(is);

        } catch (Exception e) {
            logger.warn("Error assembling system!", e);
            e.printStackTrace(System.err);
        }

    }

    //Take a path a normalize it... eg. make WEB-INF/somefile.xml and /WEB-INF/somefile.xml equivalent
    private static String getNormalizedPath(String filePath) {
        if (filePath!=null) {
            if (filePath.startsWith("/")) filePath = (filePath.length() > 1) ? filePath.substring(1) : "";
        }
        return filePath;
    }

    //--------------- DefaultHandler -----------------------------
    public class AssemblerXMLReader extends DefaultHandler {
        String parserClass = null;
        ObjectRepository or = null;
        Stack objStack = null;
        boolean needPropVal = false;
        String className = null;
        String propName = null;
        String propVal = null;
        int depth = 0;
        Map objMap = new HashMap();
        List argList = new ArrayList();
        String methodName = null;
        String returnName = null;
        int paramCntr = -1;
        boolean localContinueOnErr = false; //csc_041603.2
        String skipUntilTagName = null;     //csc_041603.2

        XMLReader parser = null;            //Since the 'ProcessXml' method can be called multiple times,
                                            // a single instance of the XmlReader is needed.

        public AssemblerXMLReader() {
            objMap.put(THIS, ObjectRepositoryAssembler.this);
        }

        public AssemblerXMLReader(String parserClass, ObjectRepository ior) {
            objMap.put(THIS, ObjectRepositoryAssembler.this);
            setParserClass(parserClass);
            setObjectRepository(ior);
        }

        /**
         * Sets the SAX Parser
         * @param parserClass the SAX parser class (if null, defaults to DEFAULT_PARSER)
         */
        public void setParserClass(String iparserClass) {
            this.parserClass = iparserClass;
        }

        /**
         * Sets the ObjectRepository that should be used
         * @param ior the object repository
         */
        public void setObjectRepository(ObjectRepository ior) {
            this.or = ior;
        }

        /**
         * Initialization stuff... (Need a better description here!)
         */
        public void setup() {
            try {
                if (parserClass==null) parserClass = DEFAULT_PARSER;
//                logger.info("Instantiating parser ("+parserClass+")");
//                parser = (XMLReader) Thread.currentThread().getContextClassLoader().loadClass(parserClass).newInstance();
                SAXParserFactory spf = SAXParserFactory.newInstance();
                logger.info("Using sax parser factory "+spf);
                spf.setNamespaceAware(true);
                parser = spf.newSAXParser().getXMLReader();
                logger.info("Using sax parser impl "+parser);
            } catch (Exception e) {
                logger.warn("Error assembling system!", e);
                e.printStackTrace(System.err);
            }
        }

        //This is the core method...
        public void processXmlFile(InputStream is) {
            try {
                //get the source file
                InputSource source = new InputSource(is);

                //parse the file
                logger.info("Assembling source file...");
                parser.setContentHandler(this);
                parser.setErrorHandler(this);
                parser.parse(source);
                logger.info("Assembly complete!");

            } catch (org.xml.sax.SAXParseException spe) {
                logger.warn("Error assembling system!", spe);
                spe.printStackTrace(System.err);
            } catch (org.xml.sax.SAXException se) {
                if (se.getException() != null) {
                    logger.warn("Error assembling system!", se.getException());
                    se.getException().printStackTrace(System.err);
                } else {
                    logger.warn("Error assembling system!", se);
                    se.printStackTrace(System.err);
                }
            } catch (Exception e) {
                logger.warn("Error assembling system!", e);
                e.printStackTrace(System.err);
            }
        }

        public void startDocument() {
            objStack = new Stack();
        }

        public void startElement(String uri, String local, String raw, Attributes attrs) throws SAXException {
            String curTag = local;
            logger.debug("Starting w/: " + curTag);
            logger.debug("uri:"+uri+" local:"+local+" raw:"+raw+" attrs:"+attrs);

            //csc_041603.2_start
            if (skipUntilTagName!=null) {
                logger.warn("Skipping start tag <"+curTag+"> because of error handling <"+skipUntilTagName+">");
                return;
            }
            String continue_on_err = attrs.getValue(CONTINUE_ON_ERR);
            localContinueOnErr = (continue_on_err!=null && 
                                  (continue_on_err.toLowerCase().equals("true") ||
                                   continue_on_err.toLowerCase().equals("yes") ||
                                   continue_on_err.toLowerCase().equals("1")));
            boolean continueOnErr = (globalContinueOnErr || localContinueOnErr);
            //csc_041603.2_end


            //object - we need to create a new object and store it in the object map
            if (curTag.equals(OBJECT)) {

                //start by getting the attrs and any args
                depth++;
                String objname = attrs.getValue(NAME);
                String clname = attrs.getValue(CLASS);
                Object arg = resolve(attrs.getValue(ARG));
                
                //if you reference this, the class and arg attributes should be ignored
                if (THIS.equals(objname)) {
                    clname = null;    
                    arg = null;
                    
                //otherwise, go ahead and build the arglist
                } else {
                    if (arg!=null) argList.add(arg);
                    for (int i=0; i<10; i++) {
                        arg = resolve(attrs.getValue(ARG+i));
                        if (arg!=null) argList.add(arg);
                    }
                }
                
                //first see if we can get the object from the map (this makes it possible
                //to reference a previously instantiated object). We only do this, however, if the
                //class name is NOT specified
                Class cl = null;
                Object obj = null;
                if (objname!=null && clname==null) obj = objMap.get(objname);
                if (obj!=null) {
                    cl = obj.getClass();
                    logger.info("Using existing obj "+obj.getClass().getName()+"@"+Integer.toHexString(obj.hashCode()));
                }

                //now try and instantiate it
                if (obj==null) try {
                    if (logger.isDebugEnabled()) {
                        logger.debug("Trying to create new --> objname:"+objname+" clname:"+clname);
                        if (argList.size()>0) {
                            String s = "    args:";
                            String sep = "";
                            for (int i=0; i<argList.size(); i++) {
                                s = s+sep+argList.get(i);
                                sep = ", ";
                            }
                            logger.debug(s);
                        }
                    }

                    //jrk_20020414.2_start
                    //Class.forName is java1. Since Java2, use
                    //Thread.currentThread().getContextClassLoader().loadClass("MyClass")
                    //This should fix some classloading issues in Engines with multiple
                    //class loaders (eg.. Tomcat-3.3.x and Tomcat-4.x.x)
                    //cl = Class.forName(clname);
                    cl = Thread.currentThread().getContextClassLoader().loadClass(clname);
                    //jrk_20020414.2_end

                    if (argList.size()<1) {
                        try {
                            obj = cl.newInstance();
                            logger.info("Instantiating "+obj.getClass().getName());
                        
                        //this err occurs when the class has no public constructor; in that case, we 
                        //want to save a reference to the class, and then only allow static methods to
                        //be invoked
                        } catch (IllegalAccessException e) {
                            obj = cl;
                            logger.info("Creating handle to class "+cl.getName()+" (for subsequent static invocation)");
                        }
                    } else {
                        Object args[] = new Object[argList.size()];
                        Class argcl[] = new Class[argList.size()];
                        for (int i=0; i<argList.size(); i++) {
                            args[i] = argList.get(i);
                            argcl[i] = argList.get(i).getClass();
                        }

                        if (logger.isDebugEnabled()) {
                            StringBuffer sb = new StringBuffer("Looking for constructor "+clname+"{");
                            String sep = "";
                            for (int i=0; i<argList.size(); i++) {
                                sb.append(sep+argcl[i].getName());
                                sep = ", ";
                            }
                            sb.append("}");
                            logger.debug(sb.toString());
                        }
                        Constructor constructor = null;
                        try {
                            constructor = cl.getConstructor(argcl);
                        } catch (Exception e) {
                        }
                        obj = constructor.newInstance(args);
                        logger.info("Instantiating "+obj.getClass().getName()+"@"+Integer.toHexString(obj.hashCode()));
                    }
                } catch (Exception e) {
                    String msg = "Error instantiating object";
                    logger.warn(msg+": ", e);
                    if (continueOnErr) {
                        logger.warn("Skipping tag <"+curTag+"> because of error...");
                        skipUntilTagName = curTag;
                        return;
                    } else {
                        throw new SAXException(msg, e);
                    }
                }

                //push the object onto the stack
                objStack.push(obj);

                //finally, if there was a name register the object accordingly
                if (objname!=null) {
                    objMap.put(objname, obj);
                }

            //method - invoke a method
            } else if (curTag.equals(METHOD)) {
                methodName = attrs.getValue(NAME);
                returnName = attrs.getValue(RETURN);
                Object arg = resolve(attrs.getValue(ARG));
                if (arg!=null) argList.add(arg);
                for (int i=0; i<10; i++) {
                    arg = resolve(attrs.getValue(ARG+i));
                    if (arg!=null) argList.add(arg);
                }
                propVal = null;
                needPropVal = true;
                paramCntr = -1;

            //csc_052902.1 - added
            //prop - set a property
            } else if (curTag.equals(PROP)) {
                propName = attrs.getValue(NAME);
                propVal = null;
                needPropVal = true;
                paramCntr = -1;

            //param - parameter (add to args list)
            } else if (curTag.equals(PARAM)) {
                //the first time we hit a param tag after a method tag,
                //reset the argList (because it may contain a blank parameter)
                if (++paramCntr==0) argList = new ArrayList();
                propVal = null;
                needPropVal = true;

            //register - register an object in the repository
            } else if (curTag.equals(REGISTER)) {
                String key = attrs.getValue(KEY);
                Object val = resolve(attrs.getValue(VAL));
                String qt = (val instanceof String || val instanceof StringBuffer ? "'" : "");
                logger.info("Registering "+key+" = "+qt+val+qt+" in the object repository");
                or.putState(key, val);
            }
        }

        public void endElement(String uri, String local, String raw) throws SAXException {
            String curTag = local;

            //csc_041603.2_start
            if (skipUntilTagName!=null) {
                if (skipUntilTagName.equals(curTag)) {
                    logger.warn("Found end of error tag <"+skipUntilTagName+">");
                    skipUntilTagName = null;
                } else {
                    logger.warn("Skipping end tag <"+curTag+"> because of error handling <"+skipUntilTagName+">");
                }
                return;
            }
            boolean continueOnErr = (globalContinueOnErr || localContinueOnErr);
            //csc_041603.2_end

            //object
            if (curTag.equals(OBJECT)) {
                //pop the object back off the stack
                objStack.pop();
                argList = new ArrayList();
                depth--;

            //param
            } else if (curTag.equals(PARAM)) {

            //method - here's where we actually invoke the method
            } else if (curTag.equals(METHOD)) {
                StringBuffer mthsb = new StringBuffer(methodName+"(");
                if (argList.size()>0) {
                    String s = "    args:";
                    String sep = "";
                    for (int i=0; i<argList.size(); i++) {
                        mthsb.append(sep+argList.get(i));
                        sep = ", ";
                    }
                    mthsb.append(")");
                }
                logger.debug("Invoking method--> "+mthsb.toString());

                //get the target object
                Object targetObj = objStack.peek();
                List clList = new ArrayList();
                List mthList = new ArrayList();
                Class ocl = (targetObj instanceof Class ? (Class) targetObj : targetObj.getClass());
                while (true) {
                    Method allMethods[] = ocl.getDeclaredMethods();
                    for (int i=0; i<allMethods.length; i++) {
                        Method m = allMethods[i];
                        if (!m.getName().equals(methodName)) continue;
                        int mod = m.getModifiers();
                        if (!Modifier.isPublic(mod)) continue;
                        Class paramCl[] = m.getParameterTypes();
                        if (paramCl.length!=argList.size()) continue;
                        logger.debug("found possible method:"+m+" paramCl:"+paramCl+" paramCl.length:"+paramCl.length);
                        if (paramCl!=null && paramCl.length>0 && paramCl[0]==String.class) mthList.add(0, m);    //methods which take String params should be given preference (since we're already dealing with String data)
                        else mthList.add(m);
                        clList.add(ocl);
                    }
                    if (ocl==Object.class) break;
                    ocl = ocl.getSuperclass();
                }
                if (mthList.size()<1) {
                    String msg = "Unable to find matching method: "+mthsb.toString();
                    logger.warn(msg);
                    if (continueOnErr) {
                        logger.warn("Skipping tag <"+curTag+"> because of error...");
                        skipUntilTagName = curTag;
                        return;
                    } else {
                        throw new SAXException(msg);
                    }
                }

                //now we need to iterate through the methods we found and see if we
                //can invoke them
                boolean success = false;
                methodsIterate: for (int i=0; i<mthList.size(); i++) {
                    //start by trying to convert the params into the target types
                    Method m = (Method) mthList.get(i);
                    logger.debug("trying so see if we can invoke method "+m);
                    Class paramTypes[] = m.getParameterTypes();
                    Object args[] = new Object[paramTypes.length];
                    for (int j=0; j<argList.size(); j++) {
                        Object arg = argList.get(j);
                        if (arg instanceof String) {
                            String sarg = (String) arg;
                            Class targetParamCl = null;
                            try {
                                if (paramTypes[j]==int.class) {
                                    targetParamCl = Integer.class;
                                    args[j] = new Integer(sarg);
                                } else if (paramTypes[j]==short.class) {
                                    targetParamCl = Short.class;
                                    args[j] = new Short(sarg);
                                } else if (paramTypes[j]==long.class) {
                                    targetParamCl = Long.class;
                                    args[j] = new Long(sarg);
                                } else if (paramTypes[j]==double.class) {
                                    targetParamCl = Double.class;
                                    args[j] = new Double(sarg);
                                } else if (paramTypes[j]==float.class) {
                                    targetParamCl = Float.class;
                                    args[j] = new Float(sarg);
                                } else {
                                    args[j] = arg;
                                }
                            } catch (Exception e) {
                                logger.debug("error trying to cast arg"+j+" to "+paramTypes[j]+"...trying next method");
                            }
                        } else {
                            args[j] = arg;
                        }
                    }
                    
                    //now try and invoke the method
                    try {
                        logger.info("...Invoking "+targetObj.getClass().getName()+"@"+Integer.toHexString(targetObj.hashCode())+"."+mthsb.toString());
                        Object o = m.invoke(targetObj, args);
                        if (returnName!=null) {
                            logger.debug("...Saving reference "+returnName+" to "+(o!=null ? o.getClass().getName()+"@"+Integer.toHexString(o.hashCode()) : "null"));
                            objMap.put(returnName, o);
                        }
                        success = true;
                        logger.debug("successfully invoked: "+m);
                        break;
                    } catch (Exception e) {
                        logger.debug("error invoking method m:"+e);
                        continue;
                    }
                }
                if (!success) {
                    String msg = "Unable to invoke method: "+mthsb.toString();
                    logger.warn(msg);
                    if (continueOnErr) {
                        logger.warn("Skipping tag <"+curTag+"> because of error...");
                        skipUntilTagName = curTag;
                        return;
                    } else {
                        throw new SAXException(msg);
                    }
                }

                //make sure we clean up the argList
                argList = new ArrayList();

            //csc_052902.1 - added
            //prop - here's where we actually set the property
            } else if (curTag.equals(PROP)) {
                logger.debug("Setting prop--> "+propName+"="+propVal);

                //get the target object
                Object targetObj = objStack.peek();
                Field fld = null;
                Class ocl = targetObj.getClass();
                while (true) {
                    Field allFields[] = ocl.getDeclaredFields();
                    for (int i=0; i<allFields.length; i++) {
                        Field f = allFields[i];
                        if (f.getName().equals(propName) && Modifier.isPublic(f.getModifiers())) {
                            logger.debug("found property:"+f);
                            fld = f;
                            break;
                        }
                    }
                    if (ocl==Object.class) break;
                    ocl = ocl.getSuperclass();
                }
                if (fld==null) {
                    String msg = "Unable to find matching property: "+propName;
                    logger.warn(msg);
                    if (continueOnErr) {
                        logger.warn("Skipping tag <"+curTag+"> because of error...");
                        skipUntilTagName = curTag;
                        return;
                    } else {
                        throw new SAXException(msg);
                    }
                }

                //now we need to set the property
                try {
                    Object iprop = fld.get(targetObj);
                    Class targetClass = (iprop!=null ? iprop.getClass() : String.class);
                    logger.debug("targetObj:"+targetObj+" iprop:"+iprop+" targetClass:"+targetClass);
                    Object targetProp = null;
                    if (propVal!=null) {
                        //..String
                        if (targetClass.equals(String.class)) {
                            targetProp = new String(propVal);

                        //..Integer
                        } else if (targetClass.equals(Integer.class)) {
                            targetProp = new Integer(propVal);

                        //..Short
                        } else if (targetClass.equals(Short.class)) {
                            targetProp = new Short(propVal);

                        //..Long
                        } else if (targetClass.equals(Long.class)) {
                            targetProp = new Long(propVal);

                        //..Double
                        } else if (targetClass.equals(Double.class)) {
                            targetProp = new Double(propVal);

                        //..Float
                        } else if (targetClass.equals(Float.class)) {
                            targetProp = new Float(propVal);

                        //..Boolean
                        } else if (targetClass.equals(Boolean.class)) {
                            String tpropVal = propVal.toLowerCase().trim();
                            targetProp = new Boolean(tpropVal.equals("true") || tpropVal.equals("yes") || tpropVal.equals("on") || tpropVal.equals("1"));
                        }
                    }
                    String bracket = (propVal instanceof String ? "\"" : "");
                    logger.info("...Setting "+fld.getDeclaringClass().getName()+"."+propName+" = "+bracket+targetProp+bracket+" ("+targetClass.getName()+")");
                    fld.set(targetObj, targetProp);

                } catch (IllegalAccessException e) {
                    String msg = "Unexpected IllegalAccessException:"+e;
                    logger.warn(msg);
                    if (continueOnErr) {
                        logger.warn("Skipping tag <"+curTag+"> because of error...");
                        skipUntilTagName = curTag;
                        return;
                    } else {
                        throw new SAXException(msg);
                    }
                }
            }

            logger.debug("Finished w/: " + curTag);
        }

        public void characters(char ch[], int start, int length) {
            //csc_041603.2 - added
            if (skipUntilTagName!=null) return;

            if (needPropVal) {
//System.out.println("ch:"+ch+" start:"+start+" length:"+length+" ch.length:"+ch.length);
//System.out.println("str:'"+new String(ch, start, length)+"'");
//System.out.println("xml:'"+XMLUtil.fromXMLUnicodeString(new String(ch, start, length))+"'");
                propVal = XMLUtil.fromXMLUnicodeString(new String(ch, start, length)).trim();
                if (propVal!=null) argList.add(propVal);
                logger.debug("got propVal:"+propVal);
                needPropVal = false;
            } else {
//                logger.debug("[characters] " + new String(ch, start, length));
            }
        }

        public void ignorableWhitespace(char ch[], int start, int length) {
//            logger.debug("[whitespace] " + new String(ch, start, length));
        }

        public void warning(SAXParseException ex) {
            System.err.println("[Warning] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        public void error(SAXParseException ex) {
            System.err.println("[Error] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        public void fatalError(SAXParseException ex) throws SAXException {
            System.err.println("[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage());
        }

        private String getLocationString(SAXParseException ex) {
            StringBuffer str = new StringBuffer();
            String systemId = ex.getSystemId();
            if (systemId!=null) {
                int index = systemId.lastIndexOf(47);
                if (index!=-1) systemId = systemId.substring(index + 1);
                str.append(systemId);
            }
            str.append(':');
            str.append(ex.getLineNumber());
            str.append(':');
            str.append(ex.getColumnNumber());
            return str.toString();
        }

        protected Object resolve(Object id) {
            if (id==null) return null;
            if (id.equals("null")) return null;
            if (objMap.containsKey(id)) {
                return objMap.get(id);
            } else {
                return id.toString();
            }
        }
    }


    //--------------- HttpServlet --------------------------------
    /**
     * This class extends HttpServlet primarily for convenience, so that you easily
     * use it to set up a servlet environment. The servlet methods that handle requests
     * do not do anything; it all happens in the config.
     */
    public void init() throws ServletException {
        logger.info("Attempting to setup default ObjectRepository (HTTP interface)");
        String descriptor = getInitParameter(ASSEMBLY_DESCRIPTOR);
        String parser = getInitParameter(SAX_PARSER);
        String hbparm = getInitParameter(LOG_HEARTBEAT_STR);
        if (hbparm!=null) {
            hbparm = hbparm.toLowerCase();
            logHeartbeat = (hbparm.equals("true") || hbparm.equals("yes") ||
                            hbparm.equals("on") || hbparm.equals("1"));
        }
        //csc_041603.2_start
        String continue_on_err = getInitParameter(GLOBAL_CONTINUE_ON_ERR);
        globalContinueOnErr = (continue_on_err!=null && 
                               (continue_on_err.toLowerCase().equals("true") ||
                                continue_on_err.toLowerCase().equals("yes") ||
                                continue_on_err.toLowerCase().equals("1")));
        //csc_041603.2_end
        this.assemble(null, this, descriptor, parser);
    }


    public static void main(String[] args) {
        //manually configure the log4j stuff
        DOMConfigurator.configure("../WEB-INF/log4j.xml");

        //assemble into the default ObjectRepository (you may need to put the
        //xml file in the classes directory in order to find it if you are
        //running this standalone)

        logger.info("Attempting to setup default ObjectRepository (Application interface)");
        new ObjectRepositoryAssembler().assemble("../WEB-INF/object-repository.xml");
        logger.info("Assembly complete!");

        //check the prop value
        System.out.println("TEST_STRING:"+TEST_STRING);
        System.out.println("TEST_INT:"+TEST_INT);
        System.out.println("TEST_SHORT:"+TEST_SHORT);
        System.out.println("TEST_LONG:"+TEST_LONG);
        System.out.println("TEST_DOUBLE:"+TEST_DOUBLE);
        System.out.println("TEST_FLOAT:"+TEST_FLOAT);
        System.out.println("TEST_BOOLEAN:"+TEST_BOOLEAN);
        System.out.println("TEST_INT2:"+TEST_INT2);
        System.out.println("TEST_SHORT2:"+TEST_SHORT2);
        System.out.println("TEST_LONG2:"+TEST_LONG2);
        System.out.println("TEST_DOUBLE2:"+TEST_DOUBLE2);
        System.out.println("TEST_FLOAT2:"+TEST_FLOAT2);
        System.out.println("TEST_BOOLEAN2:"+TEST_BOOLEAN2);

        //check and make sure it's there
        ObjectRepository or = ObjectRepository.getGlobalRepository();
        List keys = or.getStateKeys();
        if (keys!=null) {
            Iterator it = keys.iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object val = or.getState(key);
                System.out.println("Key:"+key+" Val:"+val);
            }
        }
    }

}
