/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletRequestParameterStateMap.java,v 1.2 2003/05/28 16:07:09 christianc Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The implementation provides a StateMap bridge to a ServletRequest
 * object's paramter values. By this, we mean that this class allows 
 * you to access ServletRequest paramters through the StateMap interface. 
 * Unlike the ServletRequest parameters, this class will handle null keys, 
 * values. Note however, that it's read-only. Put requests are silently ignored.
 */
public class ServletRequestParameterStateMap implements StateMap {

    //private vars
    protected ServletRequest request = null;
    private String NULL = "~Null~";                    
    
    /**
     * Public constructor.
     *
     * @param request the underlying servlet request structure
     */    
    public ServletRequestParameterStateMap(ServletRequest irequest) {
        request = irequest;
    }
    
    /**
     * set a property in this StateMap - not implemented because 
     * the servlet API doesn't allow you to set parameters
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        throw new UnsupportedOperationException();  //csc_052803_2
    }
    
    /**
     * get a property in this StateMap. Since the underlying structure is
     * an HttpRequest parameter we have to handle the case where there are 
     * multiple values for a key. In this case, a List will be returned.
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (key==null) key = NULL;
        Object[] vals = request.getParameterValues(key.toString());
        if (vals==null) return null;
        else if (vals.length==1) return (vals[0].equals(NULL) ? null : vals[0]);
        else {
            List list = new ArrayList();
            for (int i=0, max=vals.length; i<max; i++) {
                list.add(vals[i]);
            }
            return list;
        }
    }
    
    /**
     * remove a property in this StateMap - not implemented because 
     * the servlet API doesn't allow you to set parameters
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
//csc_052803_2        return null;
        throw new UnsupportedOperationException();  //csc_052803_2
    }
    
    /**
     * get a List of the keys for this StateMap (implementation
     * is an ArrayList)
     *
     * @return a List the keys for this StateMap
     */    
    public List getStateKeys() {
        Enumeration e = request.getParameterNames();
        List list = new ArrayList();
        while (e.hasMoreElements())    {
            list.add(e.nextElement());
        }        
        return list;
    }
    
    /**
     * get a copy of the underlying Map (in this case, we
     * map all the attributes of the ServletRequest structure
     * into a Map and return that)
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        Map map = new HashMap();
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = getState(key);
            map.put(key,val);            
        }
        return map;        
    }

    //csc_052803_2 - added
    /**
     * clear all state information
     */
    public void clearState() {
        throw new UnsupportedOperationException();
    }

    /**
     * get a reference to the underlying ServletRequest
     *
     * @return a reference to the underlying ServletRequest
     */    
    public ServletRequest getRequest() {
        return request;
    }
}
