/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletRequestStateMap.java,v 1.2 2003/05/28 16:07:09 christianc Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The implementation provides a StateMap bridge to a ServletRequest
 * object. By this, we mean that this class allows you to treat an
 * ServletRequest as a StateMap. Unlike the ServletRequest, this class 
 * will handle null keys, values.
 */
public class ServletRequestStateMap implements StateMap {

    //private vars
    protected ServletRequest request = null;
    private String NULL = "~Null~";                    
    
    /**
     * Public constructor.
     *
     * @param request the underlying servlet request structure
     */    
    public ServletRequestStateMap(ServletRequest irequest) {
        request = irequest;
    }
    
    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        if (key==null) key = NULL;
        if (val==null) val = NULL;
        request.setAttribute(key.toString(),val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (key==null) key = NULL;
        Object val = request.getAttribute(key.toString());
        return (val.equals(NULL) ? null : val);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        if (key==null) key = NULL;
        Object val = request.getAttribute(key.toString());
        request.removeAttribute(key.toString());
        return (val.equals(NULL) ? null : val);
    }
    
    /**
     * get a List of the keys for this StateMap (implementation
     * is an ArrayList)
     *
     * @return a List the keys for this StateMap
     */    
    public List getStateKeys() {
        Enumeration e = request.getAttributeNames();
        List list = new ArrayList();
        while (e.hasMoreElements())    {
            list.add(e.nextElement());
        }        
        return list;
    }
    
    /**
     * get a copy of the underlying Map (in this case, we
     * map all the attributes of the ServletRequest structure
     * into a Map and return that)
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        Map map = new HashMap();
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = request.getAttribute(key.toString());
            map.put(key,val);            
        }
        return map;        
    }

    //csc_052803_2 - added
    /**
     * clear all state information
     */
    public void clearState() {
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            removeState(key);
        }
    }

    /**
     * get a reference to the underlying ServletRequest
     *
     * @return a reference to the underlying ServletRequest
     */    
    public ServletRequest getRequest() {
        return request;
    }
}
